#!/bin/sh
#-----------------------------------------------------------------------
#
# Tivoli Enterprise Data Warehouse, v 1.1.2
# (C) Copyright IBM Corp. 2002.  All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#
#
#-----------------------------------------------------------------------
# File Information: src/install/bin/tedw_wpack_patchadm.sh, twh_core, twh_1.1_ode, 20021113A
#
# Revision: 1.5
#
# Date: 11/13/02 08:47:39
#-----------------------------------------------------------------------
#
# Desc: This program will launch the TWH WPack patching main program.
#
# Interpreter Required:
#       NT/2000         = BASH
#       Unix            = SH (Bourne Shell)
#
#
#########################################################################
#
#clear

# Display help that is specific to this WEP patch driver program

display_driver_help()
{
DISPLAY_HELP_ACTIVE=$TRUE;

echo ""
echo "  Program:  $driver_progname"
echo "  Desc:     This is the main utility to patch components of application"
echo "            warehouse enablement packs that are used with Tivoli"
echo "            Enterprise Data Warehouse 1.1"
echo ""
echo "  Usage:    $driver_progname [ -c config_file [ -d ] |"
echo "                                   -l [ -c config_file] [ -d ] |"
echo "                                   -d | -v | -h | -r ]"
echo ""
echo "  Options:"
echo "            -c, --config       = To specify a config file other"
echo "                                    than the default config file."
echo "                                    Specify the absolute path to"
echo "                                    configuration file."
echo "            -v, --version      = Version Information"
echo "            -h, --help         = Help Information"
echo "            -r, --retcode      = Return Code messages Information"
echo "            -d, --debug        = Run with Debug turned on"
echo "            -l, --listversions = List installed products and patches"
echo ""
echo "               Note: The '-c', '-l', and '-d' options are ignored"
echo "                     if any of the '-v', '-h', or '-r' options are"
echo "                     specified.  Only the first specified of the"
echo "                     '-v', '-h', or '-r' options are used, any"
echo "                     others are ignored."
echo ""
echo "  Default Config File:"
echo "            $TEMP/twh_app_patcher.cfg"
echo ""
echo "               Note: This file is automatically generated if does not"
echo "                     already exist and this program is called with:"
echo "                       - no parameters or options"
echo "                       - with the '-l' option but not the '-c' option"
echo "                       - with the '-d' option but not the '-c' option"
echo "                     It is not generated if the '-v', '-h', or '-r'"
echo "                     option is specified."
echo ""
echo "  Message Levels:"
echo "            (I) - Informational"
echo "            (W) - Warning"
echo "            (F) - Fatal"
echo ""

return 0
}

###############################################################
############## Driver Code Section ############################
###############################################################

# get the name of this driver program

driver_program=$0
gen_file_and_path=`echo $0 | sed 's/\\\\/\//g' `        # NT fix
driver_progname=`basename "$gen_file_and_path"`

# set conditional test constants

TRUE=1
FALSE=0

# Verify and set up path to the TEDW install on this system

if [ -z "$TWH_TOPDIR" ]; then
  echo "   !! Error - Could not locate environment variable TWH_TOPDIR !!"
  echo "      This variable points to the top of the TEDW product tree."
  echo ""
  echo "      Exiting..."
  exit 1
fi

# fix the topdir path on NT

TWH_TOPDIR=`echo "$TWH_TOPDIR" | sed 's/\\\\/\//g' `

# Verify and set up path to temp space for logging and default config file

if [ "$TEMP" = "" ]; then
  # Assume unix
  TEMP="/tmp"; export TEMP
else
  TEMP=`echo "$TEMP" | sed 's/\\\\/\//g' `
  export TEMP
fi

if [ ! -d "$TEMP" -a ! -w "$TEMP" ]; then
  echo " !! Error - $TEMP directory not found/writeable."
  echo " Install program requires the TEMP environment variable to write the"
  echo " install log to."
  exit 1
fi

# Set the default config file name

default_cfgfile="$TEMP/twh_app_patcher.cfg"

# Add directory to TEDW install tools to the PATH for the execution
#   of this program

orig_path="$PATH"
PATH="$TWH_TOPDIR/tools/bin:$PATH"; export PATH

# filter the arguments, if any, and build the default config file
#   template file if necessary


conf_val_found=$FALSE
debug_found=$FALSE
ret_code_found=$FALSE
list_vers_found=$FALSE

while [ $# -ne 0 ]; do
  # Now look for the command line arguments
  case $1 in

        # Application patch config file has been specified, save to pass on

        --config  | --CONFIG | --Config | -C | -c )
            # Set flag for value found
            conf_val_found=$TRUE
            shift
            cfgfile="$1"
                if [ ! -r "$cfgfile" ]; then
                  echo "      (F) Value not valid for config file name ($cfgfile)"
                  exit 131
                fi
            shift
            continue
            ;;

        # Version Info - display and exit

        --version | --VERSION | --Version | -V | -v )
            echo ""
            echo "  $driver_progname"
            echo "  Version 1.1.0"
            echo "  A Tivoli Enterprise Data Warehouse utility"
            echo "  IBM Software"
            echo ""
            exit 0
            ;;

        # Help Info - display and exit

        --help | --HELP | --Help | -H | -h )
            display_driver_help;
            exit 0
            ;;

        # Display the return code messages, break, we'll only do this

        --retcode | --RETCODE | --Retcode | -R | -r  )
            ret_code_found=$TRUE
            break
            ;;

        # Run in Debug, save to pass on

        --debug | --DEBUG | --Debug | -D | -d )
            debug_found=$TRUE
            shift
            continue
            ;;

          # List installed versions, break, we'll only do this

        --listversions | --LISTVERSIONS | --Listversions | -L | -l )
            list_vers_found=$TRUE
            shift
            continue
            ;;

        # Error - not valid option, display help and exit

        -*)
            echo "      (F) Unknown Command option ($1)"
            echo "          < Displaying Help >"
            echo ""
            display_driver_help;
            exit 132
            ;;

        # Default - anything else not an option is not valid, display help and exit

        *)
            echo "      (F) Unknown option ($1)"
            echo "          < Displaying Help >"
            echo ""
            display_driver_help;
            exit 133
            ;;
  esac
done

# Filter parameters that we pass on to the execute program

if [ $ret_code_found -eq $TRUE ]; then

   # indicate to display return codes list only
   arg_1="-r"
   arg_2=""
   arg_3=""
   arg_4=""

elif [ $list_vers_found -eq $TRUE ]; then
   # indicate to list installed versions only, using indicated or
   #   default config file, plus debug if requested

   # must always indicate a config file for "list versions"

   arg_1="-c"

   # check if a config file is specified or default to be used

   if [ $conf_val_found -eq $TRUE ]; then
      # if it is specified, use that one
      arg_2=$cfgfile
   else
      # if it is not specified, create and/or use the default
      if [ ! -r "$default_cfgfile" ]; then
         # if the default config file doesn't exist, then generate it as a
         #   template for the "list versions" process to use
         # make sure the auto generator is there
         if [ ! -r "$TWH_TOPDIR/install/bin/twh_create_wpack_cfg_file.sh" ]; then
           echo "      (F) Could not locate the auto generate program at location"
           echo "           ($TWH_TOPDIR/install/bin/twh_create_wpack_cfg_file.sh)"
           sleep 5
           exit 1
         fi
         # Generate the config file
         "$TWH_TOPDIR"/install/bin/twh_create_wpack_cfg_file.sh "$default_cfgfile" $FALSE
      fi
      arg_2=$default_cfgfile
   fi

   # indicate "list versions"

   arg_3="-l"

   if [ $debug_found -eq $TRUE ]; then
      arg_4="-d"
   else
      arg_4=""
   fi
elif [ $conf_val_found -eq $TRUE ]; then
   # indicate use specified existing patch config file, plus debug if requested
   arg_1="-c"
   arg_2=$cfgfile
   if [ $debug_found -eq $TRUE ]; then
      arg_3="-d"
   else
      arg_3=""
   fi
   arg_4=""
else
   # process for using default config file
   if [ ! -r "$default_cfgfile" ]; then
      # if the default config file doesn't exist, then generate it as a
      #   template and then exit to let the user complete it
      # make sure the auto generator is there
      if [ ! -r "$TWH_TOPDIR/install/bin/twh_create_wpack_cfg_file.sh" ]; then
        echo "      (F) Could not locate the auto generate program at location"
        echo "           ($TWH_TOPDIR/install/bin/twh_create_wpack_cfg_file.sh)"
        sleep 5
        exit 1
      fi
      # Generate the config file
      "$TWH_TOPDIR"/install/bin/twh_create_wpack_cfg_file.sh "$default_cfgfile" $TRUE
      exit 0            # exit so they can edit the config file
   else
      # assume the default config file is complete for this patch,
      #   indicate to use existing default config file, plus debug if requested
      arg_1="-c"
      arg_2=$default_cfgfile
      if [ $debug_found -eq $TRUE ]; then
         arg_3="-d"
      else
         arg_3=""
      fi
      arg_4=""
   fi
fi


# Pass on the log to check

INST_LOGDIR="$TEMP/twh_install_wpack_patcher.log"; export INST_LOGDIR

# Exec program

"$TWH_TOPDIR"/install/bin/twh_app_patcher.sh $arg_1 $arg_2 $arg_3 $arg_4 | tee "$TEMP"/twh_install_wpack_patcher.log
# Set path back


PATH="$orig_path"; export PATH

# Normal exit on completion

exit 0
