#!/bin/sh
#-----------------------------------------------------------------------
#
# Tivoli Enterprise Data Warehouse, v 1.1.0
# (C) Copyright IBM Corp. 2002.  All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#
#
#-----------------------------------------------------------------------
#
# File Information: src/install/bin/twh_core_after.sh, twh_core, twh_1.1_ode
#
# Revision: 1.2
#
# Date: 8/23/02 10:06:18
#-----------------------------------------------------------------------
#
# Command Line Args:
#       None.
#
# Required Config File:
#       twh_app_install.cfg
#
# Installation Logs (Default names):
#       A log will be generated for this scirpt called
#       <ava>_after.sh.log
#
#
# Required Env Vars:
#       TWH_TOPDIR
#	  TWH_CONFIG_FILE
#	  APP_WORK_DIR
#
# Return Codes:
#       0 = Success
#       255 = Failure
#
# Interpreter Required:
#       NT/2000 = BASH
#       Unix    = SH (Bourne Shell)
#
#
# Message Types:
#       (I) - Informational
#       (W) - Warning
#       (F) - Fatal
#
#
# Note - Caution needs to be taken for NT paths that have a space.
#        This is a limitation that requires double quoting around all
#        variables that contain a path.  Testing should be done to
#        ensure this script runs with a space in any of the paths 
#        that are used.  For example "c:/program files".
#
#  Note - In release 1.1.0 of the app installer, the after scripts
#         will only be run on the Control Server host only.
#
#######################################################################
#

#
update_selections()
{
echo "==> Updating RPI.REPORTS Selections"


echo "==> Testing connection to Control Database"
gr_test_connection "$TWH_CTRL_DB" "$COPT_CTRL_DB2USER" "$COPT_CTRL_DB2PASS";

# upddbm.sql is necessary for the workaround for 135548 (in updsel.sql) to run successfully
echo "==> Updating Control Database Configuration"
  gr_build_exec_script "$TWH_CTRL_DB" "$TWH_TOPDIR/install/sql" upddbm.sql $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
  gr_exec_sql_script "$TWH_CTRL_DB" $TRUE;

# After update must restart DB2
# Stop services first
echo "==> Stopping DB2 DWC NT services"
err_txt=`$DB2REGI 16 2>&1 `
if [ $? -ne 0 ]; then
  echo "==> (F) Failed to stop NT Service vwkernel and vwlogger"
  echo "        Error is:"
  echo "        $err_txt"
  echo "        Exiting..."
  exit 86
fi

# Stop DB2
echo "==> Stopping DB2"
err_txt=`$DB2CMD db2stop force 2>&1 `
if [ $? -ne 0 ]; then
  blah=`echo $err_txt | grep -i $sql_db_not_started `
  if [ $? -eq 0 ]; then
    :;                  # If db not started that is OK
  else
    echo "==> (F) Failed to stop DB2."
    echo "        Error is:"
    echo "        $err_txt"
    echo "        Exiting..."
    exit 87
  fi
fi

# Start DB2
echo "==> Starting DB2"
err_txt=`$DB2CMD db2start 2>&1 `
# Removed for defect 131629
#if [ $? -ne 0 ]; then
#  echo "      (F) Failed to start DB2."
#  echo "          Error is:"
#  echo "          $err_txt"
#  echo "          Exiting..."
#  exit 88
#fi

# Have to do this again in case db2bp thread was forced
err_txt=`$DB2EXEC_LOG connect to "$TWH_CTRL_DB" user "$COPT_CTRL_DB2USER" using "$COPT_CTRL_DB2PASS" 2>&1 `
if [ $? -ne 0 ]; then
  echo "      (F) Unable to connect to database ($TWH_CTRL_DB)."
  echo "          Error is:"
  echo "          $err_txt"
  echo "          Exiting..."
  exit 91
fi

echo "==> Testing connection to Control Database"
gr_test_connection "$TWH_CTRL_DB" "$COPT_CTRL_DB2USER" "$COPT_CTRL_DB2PASS";

# Start services (logger has to be first!)
echo "==> Starting DB2 DWC NT services"
blah=`net start vwlogger `
if [ $? -ne 0 ]; then
  echo "      (F) Failed to start NT Service vwlogger"
  echo "          Exiting..."
  exit 94
fi

blah=`net start vwkernel `
if [ $? -ne 0 ]; then
  echo "      (F) Failed to start NT Service vwkernel"
  echo "          Exiting..."
  exit 95
fi

# updsel.sql performs workarounds for 135548, 135550, and 137823
echo "==> Updating Selections in RRI.Reports"
  gr_build_exec_script "$TWH_CTRL_DB" "$TWH_TOPDIR/install/sql" updsel.sql $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
  gr_exec_sql_script "$TWH_CTRL_DB" $TRUE;

echo "==> Done updating RPI.REPORTS Selections"
return 0;
}


###


#
create_listmetric_csv()
{
metric_csv_dir="$APP_WORK_DIR/report"
metric_csv_file="rpi_listmetrics.csv"
TMP_DB_SCRIPT_FILE="$TMPDIR/tmp_exportmetrics.sql"

echo "==> Creating Metric List csv file"

conn_string="connect to $TWH_CDW_DB user $COPT_CDW_DB2USER using $COPT_CDW_DB2PASS;"
export_string="export to \"$metric_csv_dir/$metric_csv_file\" of del select msmttyp_id, msmttyp_nm from twg.msmttyp;"


# Now build the file  (gets a bit funky to put in blank lines)
echo "$conn_string" > "$TMP_DB_SCRIPT_FILE"
echo "" >> "$TMP_DB_SCRIPT_FILE"
echo "$export_string"  >> "$TMP_DB_SCRIPT_FILE"
echo "" >> "$TMP_DB_SCRIPT_FILE"

if [ ! -r "$TMP_DB_SCRIPT_FILE" ]; then
  echo "      (F) Error trying to read execution script ($TMP_DB_SCRIPT_FILE)"
  rm -f "$TMP_DB_SCRIPT_FILE"
  exit 101
elif [ -x "$TMP_DB_SCRIPT_FILE" ]; then
  chmod 777 "$TMP_DB_SCRIPT_FILE"
fi
echo "      (I) Done building metric csv script"

gr_exec_sql_script "$TWH_CDW_DB" $TRUE;

echo "==> Done creating Metric List csv file"
return 0;
}


###


#
load_ids()
{
csv_file=$1
create_table=$2
script_file=$3
text=$4

echo "==> Loading $4 ID's"


exec_dir="$TWH_TOPDIR/rpi/sql"
csv_dir="$APP_WORK_DIR/report"
csv_script="$exec_dir/$script_file"
create_table_script="$exec_dir/$create_table"
TMP_DB_SCRIPT_FILE="$TMPDIR/tmp_$script_file"

conn_string="connect to $TWH_CTRL_DB user $COPT_CTRL_DB2USER using $COPT_CTRL_DB2PASS;"
import_string="import from '$csv_dir/$csv_file' of del insert into rpi.rpi_install_tmp;"
drop_table_string="drop table rpi.rpi_install_tmp;"

echo "      (I) Building report load script for ($script_file)"
# generic create table script
if [ ! -r "$create_table_script" ]; then
  echo "      (F) Could not locate ($create_table_script)."
  exit 98
fi

# csv script
if [ ! -r "$csv_script" ]; then
  echo "      (F) Could not locate ($csv_script) file."
  exit 99
fi

# csv data file
if [ ! -r "$csv_dir/$csv_file" ]; then
  echo "      (F) Could not locate ($csv_dir/$csv_file) file."
  exit 100
fi


# Now build the file  (gets a bit funky to put in blank lines)
echo "$conn_string" > "$TMP_DB_SCRIPT_FILE"
cat "$create_table_script" >> "$TMP_DB_SCRIPT_FILE"
echo "" >> "$TMP_DB_SCRIPT_FILE"
echo "$import_string"  >> "$TMP_DB_SCRIPT_FILE"
cat "$csv_script" >> "$TMP_DB_SCRIPT_FILE"
echo "" >> "$TMP_DB_SCRIPT_FILE"
echo "$drop_table_string" >> "$TMP_DB_SCRIPT_FILE"
if [ ! -r "$TMP_DB_SCRIPT_FILE" ]; then
  echo "      (F) Error trying to read Created DDL execution script ($TMP_DB_SCRIPT_FILE)"
  rm -f "$TMP_DB_SCRIPT_FILE"
  exit 101
elif [ -x "$TMP_DB_SCRIPT_FILE" ]; then
  chmod 777 "$TMP_DB_SCRIPT_FILE"
fi
echo "      (I) Done building $text ID loading script"

gr_exec_sql_script "$TWH_CTRL_DB" $TRUE;

echo "==> Done loading $text IDs"
return 0;
}


###


#
drop_id_tables()
{
ignore_error=$1
# drops table rpi.imported_reports and rpi.report_metrics used by java program
echo "==> Dropping Selections and Metric ID table"
  gr_build_exec_script "$TWH_CTRL_DB" "$TWH_TOPDIR/rpi/sql" rpi_droplist_tables.sql $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
  gr_exec_sql_script "$TWH_CTRL_DB" $ignore_error;
  #setting to ignore error as the first time through the table most likely will not be here
  #but still need to attempt to drop first to ensure rerunability
echo "==> Done dropping Selections and Metric ID table"
return 0;
}


###


#
SelectionsUpdateDirector()
{
echo "==> Calling the Selections Update Director"
  CLASSPATH="$TWH_TOPDIR\tools\bin\twh_rpiselections_update.jar;$TWH_TOPDIR\tools\bin\com.tivoli.twh.rpi.impl.jar;$TWH_TOPDIR\tools\bin\com.tivoli.twh.rpi.nls.jar;$TWH_TOPDIR\tools\bin\com.tivoli.twh.rpi.repnav.impl.jar;$TWH_TOPDIR\tools\bin\com.tivoli.twh.utilities.jar;$TWH_TOPDIR\tools\bin\mm_util.jar;$TWH_TOPDIR\tools\bin\log.jar;$TWH_TOPDIR\tools\bin\IBMJCEfw.jar;$TWH_TOPDIR\tools\bin\IBMJCEProvider.jar;$TWH_TOPDIR\tools\bin\db2java.zip;"
  "$TWH_TOPDIR"/_jvm/bin/java -Dtwh.topdir="$TWH_TOPDIR" com.tivoli.twh.rpi.SelectionsUpdateDirector;
  rc=$?

  if [ $rc -ne 0 ]; then
    echo "      (F) Failure in Selections Update Director.  RC=($rc)"
    exit $rc
  fi

echo "==> Selections Update Director completed successfully"
return 0;
}


###


#
recycle_ps()
{

if [ ! -d "$PS_HOME" ]; then
  echo "      (F) Could not locate PS home directory ($PS_HOME)"
  exit 1
fi

# Env setup
if [ $UNIX_INTERP -eq $FALSE ]; then
  ps_exec_path="$PS_HOME/bin/w32-ix86"
  ps_exec_path_private="$PS_HOME/bin/private/w32-ix86"
  extension=".bat"
else
  ps_exec_path="$PS_HOME/bin/generic_unix"
  ps_exec_path_private="$PS_HOME/bin/private/generic_unix"
  extension=".sh"
fi

# recycle PS services

if [ "$INTERP" != "AIX4-R1" ]; then
    # Stop servers
    echo "      (I) Stopping MCR and WC servers"
    blah=`"$ps_exec_path"/stopwc${extension} 2>&1 `
    blah=`"$ps_exec_path"/stopmcr${extension} 2>&1 `
else
    # Stop servers
    # On AIX we need to stop the servers using the scripts in the
    # bin/private/generic_unix directory.  The public stop scripts
    # will not actually stop the services.
    echo "      (I) Stopping MCR and WC servers"
    blah=`"$ps_exec_path_private"/stopwc${extension} 2>&1 `
    blah=`"$ps_exec_path_private"/stopmcr${extension} 2>&1 `
fi

echo "      (I) Starting MCR server"
blah=`"$ps_exec_path"/mcr${extension} 2>&1 `

# Wait for them to start...
echo "      (I) Waiting for MCR server to start"
blah=`"$ps_exec_path_private"/waitmcr${extension} 2>&1 `
rc=$?
if [ $rc -ne 0 ]; then
  echo "      (F) Failed to run waitmcr program.  RC=($rc)"
  echo "          Error was:"
  echo "          $blah"
  exit 1
fi

echo "      (I) Starting WC server"
blah=`"$ps_exec_path"/wc${extension} 2>&1 `

return 0;
}


############
### Main ###
############

# Load local environment for this program run
. "$TWH_TOPDIR"/install/bin/twh_core_startup_after.sh
exec_startup_driver;


################
#  Driver code #
################
echo "==> Processing main Core routines"
echo ""

# Skip this if directed to do so
if [ "$REPORTS_INCLUDED" != "TRUE" ]; then
  echo "      (W) Skipping Post Installation Report Processing as requested."
  echo "          Returning control to calling program."
  exit 1;		# Return with warning
fi


# Now execute main functions
update_selections;
create_listmetric_csv;
drop_id_tables $FALSE;

load_ids "${AVA_CODE}_rpi_export3.csv" "rpi_export3_create_table.sql" "rpi_listreports_process_table.sql" Report;

load_ids "rpi_listmetrics.csv" "rpi_listmetrics_create_table.sql" "rpi_listmetrics_process_table.sql" Metric;

# Recycle PS Services. If there is a connection handle it gets re-used 
# so any new info won't be reflected on that connection.
# 148059: there's no way to recycle PS on a distributed install. We're going
#         to doc that PS must be recycled manually after an application install.
#recycle_ps;
SelectionsUpdateDirector;
drop_id_tables $TRUE;


echo "==> Completed executing script"

exit 0