#!/bin/sh
#----------------------------------------------------------------------
#
# Tivoli Enterprise Data Warehouse, v 1.1.2 
# (C) Copyright IBM Corp. 2002.  All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#
#
#----------------------------------------------------------------------
#
# File Information: src/install/bin/twh_core_patcher.sh, twh_core, twh_1.1_ode, 20021007A
#
# Revision: 1.27
#
# Date: 10/4/02 12:56:38
#-----------------------------------------------------------------------
#
# Primary Author: D. Ledden
#
#
# Desc:	This program will patch and re-configure the Tivoli
#		Warehouse Core components.
#
# Command Line Args:
#	-v = Release Version
#	-h = Help
#	-r = Show return code messages
#	-d = Run in Debug
#	-l = List installed products and patches
#
#
# Required Config File:
#	twh_core_patcher.cfg
#
#
# Required Env Vars:
#	None.
#
# Return Codes:
#	* See display_return_code_messages function below
#
#
# Interpreter Required:
#	NT/2000 	= BASH
#	Unix		= SH (Bourne Shell)
#
#
# Message Types:
#	(I) - Informational
#	(W) - Warning
#	(F) - Fatal
#
# Install Steps Required Before this Script Executes:
#
#########################################################################
#
###############################################################
################ Custom Functions Section #####################
###############################################################

# Pulls in the generic shared routines
# Note return codes 3-70 reserved for the gen routines
include_gen_routines()
{
TWH_CORE_PATCH_CONTEXT=$TRUE; export TWH_CORE_PATCH_CONTEXT

# Bug on NT - fix the path back to be unix type path
gen_file_and_path=`echo $PROGRAM | sed 's/\\\\/\//g' `

gen_path=`dirname "$gen_file_and_path" `

# Confirm file (twh_gen_routines must be in same directory)
if [ ! -r "$gen_path"/twh_gen_routines.sh ]; then
  echo "      (F) CDWIC0071E Could not locate the twh_gen_routines.sh file."
  exit 71
fi

# pull in the generic routines
. "$gen_path"/twh_gen_routines.sh


return 0;
}


###


# Display Help
display_help()
{
DISPLAY_HELP_ACTIVE=$TRUE;


echo ""
echo "  Program:  $PROG_NAME"
echo "  Desc:     This is the main program to patch the Tivoli Warehouse (TEDW)"
echo "            Core components."
echo ""
echo "  Usage:    $PROG_NAME -c config_file [ options... ]"
echo ""
if [ "$locally_called" = "$FALSE" ]; then
  echo "            Run this program without any options to auto-"
  echo "            generate a config file.  Edit the config file found"
  echo "            at ($cfg_file_loc) to include any missing properties."
  echo "            Then re-run this program as defined in the usage"
  echo "            statement above using options desired."
fi
echo ""
echo "  Options:"
echo "            -c, --config       = The required config file"
echo "            -v, --version      = Version Information"
echo "            -h, --help         = Help Inforamtion"
echo "            -r, --retcode      = Return Code messages Inforamtion"
echo "            -d, --debug        = Run with Debug turned on"
echo "            -l, --listversions = List installed products and patches"
echo ""
echo "  Config File:"
echo "            twh_core_patcher.cfg"
echo ""
echo ""
echo "  Message Levels:"
echo "		  (I) - Informational"
echo "		  (W) - Warning"
echo "		  (F) - Fatal"
echo ""

return 0
}


###


# Display return code messages - Maintain with TWHStrings.java
display_return_code_messages()
{
echo ""
echo "    Return code messages for this program:"
echo "    ======================================"

gr_display_return_codes;

echo ""
echo "    Core Install specific errors:"
echo "    71 = Could not locate the twh_generic_routines include file"
echo ""

return 0;
}


###


#
set_private_program_vars()
{
# Fixpack 1
FP1_VERSION="1.1.1"
FP1_CORE_PATCH_VERSION_FILE="$TWH_TOPDIR/.installed/.twh_core_v${FP1_VERSION}.marker"
FP1_BCOMPAT_CORE_PATCH_VERSION_FILE="$TWH_TOPDIR/.twh_core_v${FP1_VERSION}.marker"


# Current Fixpack
CORE_PATCH_VERSION_FILE="$TWH_TOPDIR/.installed/.twh_core_v${TWH_VERSION}.marker"
BCOMPAT_CORE_PATCH_VERSION_FILE="$TWH_TOPDIR/.twh_core_v${TWH_VERSION}.marker"
PATCH_LOGS_DIR="$TWH_TOPDIR/install/logs/patch_${TWH_VERSION}"


# Must launch setup.exe from interp dir in patch tree
PATCH_TOPDIR=`echo "$PATCH_MEDIA_DIR"`;  export PATCH_TOPDIR
PATCH_GENERIC_DIR="$PATCH_TOPDIR/generic"

interp_lower=`echo $INTERP | tr '[A-Z]' '[a-z]' `
PATCH_INTERP_DIR="$PATCH_TOPDIR/$interp_lower"

CORE_BACKUP_DIR="$TWH_TOPDIR/install/db_backup"

return 0
}


###


#
display_warning()
{
echo "    ============================================================="
echo "                        WARNING!!!"
echo "    You should ensure you have completed all required manual"
echo "    backups of all Databases and product directories"
echo "    associated with the TEDW product.  These procedures are defined"
echo "    in the readme.txt file provided with this patch."
echo "    Failure to do so may result in the inability to restore"
echo "    this product in the state it was in prior to this Fix Pack."
echo ""
echo "    You can Contol-C to exit"
echo "    ============================================================="
if [ "$INST_WIN_LOGDIR" != "" ]; then
  echo "    Note - A copy of this Fix Pack install log will be at ($INST_WIN_LOGDIR)"
elif [ "$INST_UNIX_LOGDIR" != "" ]; then
  INST_UNIX_LOGDIR=`dirname "$INST_UNIX_LOGDIR" `
  echo "    Note - A copy of this Fix Pack install log will be at ($INST_UNIX_LOGDIR)"
fi
echo ""

sleep 10

return 0;
}


###


validate_basic_install()
{
echo "==> Checking for basic install info"
if [ ! -r "$TWH_TOPDIR" ]; then
  echo "      (F) Could not locate the Product Top directory ($TWH_TOPDIR)"
  echo "          Exiting..." 
  echo ""
  exit 72
fi


echo "      (I) Done checking for basic installation."
return 0;
}


###


# Check that db2 is installed at the correct level.
# Parameters:
#     $1 - minimum major level desired
#     $2 - minimum minor level desired
#     $3 - minimum fixpack level desired
validate_db2_level()
{
minmajor=$1
minminor=$2
minfixpack=$3

echo "==> Validating DB2 version installed on this system."

if [ $UNIX_INTERP -eq $FALSE ]; then
  blah=`type db2level 2>&1`
  if [ $? -ne 0 ]; then
    echo "      (F) Could not find db2level command on this system.  Please verify that DB2 is installed on this"
    echo "          system at version $minmajor.$minminor.0.$minfixpack."
    exit 73
  fi
  
  levelout=`db2level`
  level=`echo $levelout | perl5 -e "\\\$levelinfo=<STDIN>; \
      @fields=split /(\")|(\s)/, \\\$levelinfo; \
      @level=grep /^v7/, @fields; \
      print \"@level\n\""`
  nodetype=`$DB2CMD db2 get dbm cfg 2>&1 | grep 'Node type'`
else
  gr_su_cmd $DB2USER "db2level"
  if [ $? -ne 0 ]; then
    echo "      (F) Could not find db2level command on this system.  Please verify that DB2 is installed on this"
    echo "          system at version $minmajor.$minminor.0.$minfixpack and that the DB2USER specified in the config"
    echo "          file is correct."
    exit 74
  fi
  level=`echo $GRSU_OUT | cut -d'"' -f8 | cut -d' ' -f2`

  gr_su_cmd $DB2USER "db2 get dbm cfg 2>&1 | grep 'Node type'"
  nodetype=$GRSU_OUT
fi

db2major=`echo $level | cut -d"." -f1 | cut -c2-`
db2minor=`echo $level | cut -d"." -f2`
db2fixpack=`echo $level | cut -d"." -f4`

if [ $db2major -lt $minmajor ]; then
  echo "      (F) Invalid db2 version detected.  DB2 should be at level $minmajor.$minminor.0.$minfixpack"
  exit 75
elif [ $db2minor -lt $minminor ]; then
  echo "      (F) Invalid db2 version detected.  DB2 should be at level $minmajor.$minminor.0.$minfixpack"
  exit 75
elif [ $db2fixpack -lt $minfixpack ]; then
  echo "      (F) Invalid db2 version detected.  DB2 should be at level $minmajor.$minminor.0.$minfixpack"
  exit 76
else
  echo "      (I) A valid db2 level has been detected."
fi

}
   
   
###


#
validate_core_comp_list_file()
{
# Validate all properties are here
if [ -z "$TWH_CORE_INSTALL_TYPE" ]; then
  echo "      (F) CDWIA0106E Error.  Property TWH_CORE_INSTALL_TYPE not found in config file"
  exit 83
elif [ -z "$TWH_CTRL_COMP" ]; then
  echo "      (F) CDWIA0106E Error.  Property TWH_CTRL_COMP not found in config file"
  exit 83
elif [ -z "$TWH_CTRL_HOST" ]; then
  echo "      (F) CDWIA0106E Error.  Property TWH_CTRL_HOST not found in config file"
  exit 83
elif [ -z "$TWH_CDW_HOST" ]; then
  echo "      (F) CDWIA0106E Error.  Property TWH_CDW_HOST not found in config file"
  exit 83
elif [ -z "$TWH_MART_HOST" ]; then
  echo "      (F) CDWIA0106E Error.  Property TWH_MART_HOST not found in config file"
  exit 83
elif [ -z "$TWH_RPI_HOST" ]; then
  echo "      (F) CDWIA0106E Error.  Property TWH_RPI_HOST not found in config file"
  exit 83
elif [ -z "$TWH_CDW_COMP" ]; then
  echo "      (F) CDWIA0106E Error.  Property TWH_CDW_COMP not found in config file"
  exit 83
elif [ -z "$TWH_MART_COMP" ]; then
  echo "      (F) CDWIA0106E Error.  Property TWH_MART_COMP not found in config file"
  exit 83
elif [ -z "$TWH_RPI_COMP" ]; then
  echo "      (F) CDWIA0106E Error.  Property TWH_RPI_COMP not found in config file"
  exit 83
elif [ -z "$TWH_DB2_NODE_MAJIC" ]; then
  echo "      (F) CDWIA0106E Error.  Property TWH_DB2_NODE_MAJIC not found in config file"
  exit 83
elif [ -z "$TWH_CTRL_NODE_ALIAS" ]; then
  echo "      (F) CDWIA0106E Error.  Property TWH_CTRL_NODE_ALIAS not found in config file"
  exit 83
elif [ -z "$TWH_CDW_NODE_ALIAS" ]; then
  echo "      (F) CDWIA0106E Error.  Property TWH_CDW_NODE_ALIAS not found in config file"
  exit 83
elif [ -z "$TWH_MART_NODE_ALIAS" ]; then
  echo "      (F) CDWIA0106E Error.  Property TWH_MART_NODE_ALIAS not found in config file"
  echo "          Ensure a carriage return is not missing from the last line in the config file"
  exit 83
fi


return 0;
}


###


#  Read the supplied core component list file
#  This file will tell us what type of core install was
#  selected and some properties if they did a Custom install
read_core_comp_list_file()
{
echo "==> Reading the Core Component List file located"
echo "    at ($TWH_TOPDIR/install/logs)."


# Is it there, we have to handle this not being here
# because the install may not have made it this far.
if [ ! -r "$TWH_CORE_COMP_FILE" ]; then
  echo "      (F) File not found. This program requires this file"
  echo "          to define what TEDW components have been installed"
  echo "          on this host."
  echo "          Exiting..."
  exit 84;
fi


# Open our config file
exec < "$TWH_CORE_COMP_FILE"


# Parse the core comp file
while :
  do
  # Read a line
  read cur_line

  # Did we hit the end of the file?
  if [ $? -ne 0 ]; then break; fi

  # clean whitespace and look for comment char
  cleaned_line=`echo "$cur_line" | sed "s/        //g" `

  # On Windows, do this to avoid ctrl M issues
  if [ "$INTERP" = "$WIN" ]; then
    blah=`echo "$cleaned_line" | grep -v '[a-zA-Z].*' `
    if [ $? -eq 0 ]; then continue; fi
  fi

  # Goto next line if this is a comment
  echo "$cleaned_line" | grep -i "^#" > /dev/null
  if [ $? -eq 0  -o ! "$cleaned_line" ]; then
    continue
  fi

  # Now parse for properties and validate them
  echo "$cleaned_line" | grep "=" > /dev/null
  if [ ! $? -eq 0 ]; then
    gr_print_conf_read_error "$cleaned_line" ;
    exit 85
  fi

  # Find the property tag and value and upcase the tag
  property_line=`echo "$cleaned_line" | sed "s/#.*//" `
  prop_tag=`echo "$property_line" | sed "s/=.*//" | tr '[a-z]' '[A-Z]' `
  prop_val=`echo "$property_line" | sed "s/.*=//" `

  # Ensure they supplied something
  if [ -z "$prop_val" ]; then
    echo "      (F) CDWIA0108E No value found for property ($prop_tag)."
    exit 86
  fi

  # Now define the property for later use
  case $prop_tag in

    #
    TWH_CORE_INSTALL_TYPE)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_CORE_INSTALL_TYPE="$prop_val"
        echo "      (I) Core installation type was ($TWH_CORE_INSTALL_TYPE)"
        uppr_val=`echo "$TWH_CORE_INSTALL_TYPE" | tr '[a-z]' '[A-Z]'`
        if [ "$uppr_val" = "TWH_CUSTOM" ]; then 
          CUSTOM_INSTALL=$TRUE
        fi
        export CUSTOM_INSTALL
        ;;

    #
    TWH_CTRL_COMP)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_CTRL_COMP="$prop_val"
        uppr_val=`echo "$TWH_CTRL_COMP" | tr '[a-z]' '[A-Z]'`
        if [ "$uppr_val" = "TRUE" ]; then 
          ADMIN_CHECKED=$TRUE
        fi
        export ADMIN_CHECKED
        ;;

    #
    TWH_CTRL_HOST)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_CTRL_HOST="$prop_val"
        export TWH_CTRL_HOST
        ;;

    #
    TWH_CDW_HOST)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_CDW_HOST="$prop_val"
        export TWH_CDW_HOST
        ;;

    #
    TWH_MART_HOST)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_MART_HOST="$prop_val"
        export TWH_MART_HOST
        ;;

    #
    TWH_RPI_HOST)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_RPI_HOST="$prop_val"
        export TWH_RPI_HOST
        ;;

    #
    TWH_CDW_COMP)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_CDW_COMP="$prop_val"
        uppr_val=`echo "$TWH_CDW_COMP" | tr '[a-z]' '[A-Z]'`
        if [ "$uppr_val" = "TRUE" ]; then 
          CDW_CHECKED=$TRUE
        fi
        export CDW_CHECKED
        ;;
    #
    TWH_MART_COMP)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_MART_COMP="$prop_val"
        uppr_val=`echo "$TWH_MART_COMP" | tr '[a-z]' '[A-Z]'`
        if [ "$uppr_val" = "TRUE" ]; then 
          MART_CHECKED=$TRUE
        fi
        export MART_CHECKED
        ;;
    #

    TWH_RPI_COMP)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_RPI_COMP="$prop_val"
        uppr_val=`echo "$TWH_RPI_COMP" | tr '[a-z]' '[A-Z]'`
        if [ "$uppr_val" = "TRUE" ]; then 
          RPI_CHECKED=$TRUE
        fi
        export RPI_CHECKED
        ;;

    TWH_DB2_NODE_MAJIC)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_DB2_NODE_MAJIC="$prop_val"
        export TWH_DB2_NODE_MAJIC
        ;;

    TWH_CTRL_NODE_ALIAS)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_CTRL_NODE_ALIAS="$prop_val"
        export TWH_CTRL_NODE_ALIAS
        ;;

    TWH_CDW_NODE_ALIAS)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_CDW_NODE_ALIAS="$prop_val"
        export TWH_CDW_NODE_ALIAS
        ;;

    TWH_MART_NODE_ALIAS)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_MART_NODE_ALIAS="$prop_val"
        export TWH_MART_NODE_ALIAS
        ;;

    # Default - is an error
    *)
        echo "      (F) CDWIA0108E Illegal option found."
        echo "          Illegal value is ($prop_tag)"
        exit 87
        ;;
    esac
  done


# Detect any missing properties
validate_core_comp_list_file;

echo "      (I) Completed reading Core Component List file"
return 0;
}


###


# Exported values out of this function:
# 	VERSION_MAJOR
# 	VERSION_MINOR
# 	VERSION_MAINT
# 	VERSION_FIXPK
# 	VERSION_REL = (GREATER, EQUAL, LESS)
# This function will determine the relationship between the installing program
# and the highest installed level.  Context for relationship is "app is (greater, equal,
# or less ) then the highest installed software version
check_version()
{
    app_name=$1
    app_ver=$2
    ignore_display="$3"			# Optional
    markers=""
    VERSION_MAJOR=0
    VERSION_MINOR=0
    VERSION_MAINT=0
    VERSION_FIXPK=0
    VERSION_REL=""
    MFDIR="$TWH_TOPDIR";
    VERSION_INSTALLED="0.0.0.0"

    if [ "$app_name" != "core" ]; then
        MFDIR="$TWH_TOPDIR"/apps/$app_name
    fi


    # Find latest installed level of app_name given.

    ls "$MFDIR"/.twh_$app_name_*.marker 2> /dev/null > "$TMPDIR/version.dat"
    ls "$MFDIR"/.installed/.twh_$app_name_*.marker 2> /dev/null >> "$TMPDIR"/version.dat


    # Solaris bug fix
    if [ "$INTERP" = "$SOL" ]; then
      rm -f "$TMPDIR"/.VERSION_MAJOR.dat "$TMPDIR"/.VERSION_MINOR.dat
      rm -f "$TMPDIR"/.VERSION_MAINT.dat "$TMPDIR"/.VERSION_FIXPK.dat

      touch "$TMPDIR"/.VERSION_MAJOR.dat
      touch "$TMPDIR"/.VERSION_MINOR.dat
      touch "$TMPDIR"/.VERSION_MAINT.dat
      touch "$TMPDIR"/.VERSION_FIXPK.dat
    fi

    while :
      do

	read i
      if [ "$i" = "" ]; then
        break;
      fi

	marker=`basename "$i"`

	# Skip this dummy record
	if [ "$marker" = ".twh_core_custom.marker" ]; then
        continue;
	fi 

	# Skip E-Fix values.  No dependency on E-Fixes
	blah=`echo $marker | grep @efix 2>&1 `
	if [ $? -eq 0 ]; then
        continue;
	fi



      # Strip marker file of everything but the version info.
	VERSION_INSTALLED=`echo $marker | sed "s/\.twh_${app_name}_v//g" | sed "s/\.marker//g" `
	VERSION_INSTALLED=`echo $VERSION_INSTALLED | cut -d"@" -f1`

      MF_MAJOR=`echo $VERSION_INSTALLED | cut -d"." -f1`
	MF_MINOR=`echo $VERSION_INSTALLED | cut -d"." -f2`
	MF_MAINT=`echo $VERSION_INSTALLED | cut -d"." -f3`
	MF_FIXPK=`echo $VERSION_INSTALLED | cut -d"." -f4`
      
  
        # Windows hack for versions that do not have the 4th digit in the version 
        if [ "$MF_FIXPK" = "$VERSION_INSTALLED" ]; then
            MF_FIXPK=0
        fi


	if [ $MF_MAJOR -gt $VERSION_MAJOR ]; then
            VERSION_MAJOR=$MF_MAJOR
	    VERSION_MINOR=$MF_MINOR
	    VERSION_MAINT=$MF_MAINT
            if [ "$MF_FIXPK" != "" ]; then
	        VERSION_FIXPK=$MF_FIXPK
            else
	        VERSION_FIXPK=0
            fi
        elif [ $MF_MAJOR -eq $VERSION_MAJOR ]; then
            if [ $MF_MINOR -gt $VERSION_MINOR ]; then
    	        VERSION_MINOR=$MF_MINOR
	        VERSION_MAINT=$MF_MAINT
                if [ "$MF_FIXPK" != "" ]; then
	            VERSION_FIXPK=$MF_FIXPK
                else
	            VERSION_FIXPK=0
                fi
            elif [ $MF_MINOR -eq $VERSION_MINOR ]; then
                if [ $MF_MAINT -gt $VERSION_MAINT ]; then
  	            VERSION_MAINT=$MF_MAINT
                    if [ "$MF_FIXPK" != "" ]; then
	                VERSION_FIXPK=$MF_FIXPK
                    else
	                VERSION_FIXPK=0
                    fi
                elif [ $MF_MAINT -eq $VERSION_MAINT ]; then
                    if [ "$MF_FIXPK" != "" ]; then
                        if [ $MF_FIXPK -gt $VERSION_FIXPK ]; then
      	                    VERSION_FIXPK=$MF_FIXPK
			fi
                    fi
                fi
            fi
        fi




    # Solaris Bug Hack.  Solaris has var scoping bug in while loops
    # Workaround is to persist vars in tmp files - ugly I know.
    if [ "$INTERP" = "$SOL" ]; then
      echo $VERSION_MAJOR > "$TMPDIR"/.VERSION_MAJOR.dat
      echo $VERSION_MINOR > "$TMPDIR"/.VERSION_MINOR.dat
      echo $VERSION_MAINT > "$TMPDIR"/.VERSION_MAINT.dat
      echo $VERSION_FIXPK > "$TMPDIR"/.VERSION_FIXPK.dat
    fi

    done < "$TMPDIR"/version.dat
    rm -f "$TMPDIR"/version.dat


    if [ "$INTERP" = "$SOL" ]; then
      VERSION_MAJOR=`cat "$TMPDIR"/.VERSION_MAJOR.dat `
      VERSION_MINOR=`cat "$TMPDIR"/.VERSION_MINOR.dat `
      VERSION_MAINT=`cat "$TMPDIR"/.VERSION_MAINT.dat `
      VERSION_FIXPK=`cat "$TMPDIR"/.VERSION_FIXPK.dat `
      
      rm "$TMPDIR"/.VERSION_MAJOR.dat "$TMPDIR"/.VERSION_MINOR.dat
      rm "$TMPDIR"/.VERSION_MAINT.dat "$TMPDIR"/.VERSION_FIXPK.dat
    fi


    # Set highest version
    VERSION_INSTALLED="${VERSION_MAJOR}.${VERSION_MINOR}.${VERSION_MAINT}.${VERSION_FIXPK}"
    if [ "$ignore_display" = "" ]; then 
      echo "      (I) Highest version installed is ($VERSION_INSTALLED)"
    fi

    if [ "$app_ver" != "" ]; then
	  breakdown_version_levels "$app_ver";

        APP_MAJOR=$MAJOR
        APP_MINOR=$MINOR
        APP_MAINT=$MAINT
        APP_FIXPK=$FIXPK

	# Fourth digit was optional
	if [ "$APP_FIXPK" = "" ]; then APP_FIXPK=0; fi

	# Set Equal, greater, less
      VERSION_REL="LESS"
	if [ $APP_MAJOR -gt $VERSION_MAJOR ]; then
            VERSION_REL="GREATER"
        elif [ $APP_MAJOR -eq $VERSION_MAJOR ]; then
            if [ $APP_MINOR -gt $VERSION_MINOR ]; then
                VERSION_REL="GREATER"
            elif [ $APP_MINOR -eq $VERSION_MINOR ]; then
                if [ $APP_MAINT -gt $VERSION_MAINT ]; then
                    VERSION_REL="GREATER"
                elif [ $APP_MAINT -eq $VERSION_MAINT ]; then
                    if [ $APP_FIXPK -gt $VERSION_FIXPK ]; then
                        VERSION_REL="GREATER"
                    elif [ $APP_FIXPK -eq $VERSION_FIXPK ]; then
                        VERSION_REL="EQUAL"
                    fi
                fi
            fi
        fi
    fi


return 0;
}


###


#
breakdown_version_levels()
{
ver_in="$1"

MAJOR=`echo $ver_in | cut -d"." -f1`
MINOR=`echo $ver_in | cut -d"." -f2`
MAINT=`echo $ver_in | cut -d"." -f3`
FIXPK=`echo $ver_in | cut -d"." -f4`

#Fourth digit was optional
if [ "$FIXPK" = "" ]; then FIXPK=0; fi


return 0;
}


###


# Check all products and patches to see if we can install this
validate_patch_installable()
{
echo "==> Validating this Fix Pack version can be installed."


breakdown_version_levels "$TWH_VERSION";
check_version "core" "$TWH_VERSION";

# Set fixpack values
CORE_LATEST_FIXPACK_VER=$VERSION_FIXPK; export CORE_LATEST_FIXPACK_VER
CORE_FIXPACK_VER=$FIXPK; export CORE_FIXPACK_VER


# Can then install this?
if [ "$VERSION_REL" = "GREATER" ]; then
  # ensure it is the correct release chain
  if [ "$MAJOR" = "$VERSION_MAJOR" -a "$MINOR" = "$VERSION_MINOR" ]; then
    echo "      (I) Fix Pack version (2) validated and can be installed."
  else
    echo "      (F) You are not installing this fixpack ($TWH_VERSION)"
    echo "          on the correct release chain."
    echo "          Exiting..."
    exit 1
  fi
elif [ "$VERSION_REL" = "EQUAL" ]; then
    echo "      (F) TEDW Fix Pack Version (2) is already detected on this system." 
    echo "          To prevent destroying an existing installation,"
    echo "          you cannot reinstall.  You must explicitly"
    echo "          remove this Fix Pack first if you want to reinstall it."
    echo "          Exiting..."
    exit 1
else
  echo "      (F) The Core version installed on this host is newer than"
  echo "          the version ($TWH_VERSION) that you are trying to"
  echo "          install."
  echo "          Exiting..."
  exit 1
fi


echo "      (I) Done validating Fix Pack version"
return 0;
}


###


execute_md_db_backup()
{
echo "          Making backup of the database before making updates to the DWC MD Database."

sql_db_not_started="SQL1032N"


# Stop services first
echo "      (I) Stopping DB2 DWC NT services"
err_txt=`$DB2REGI 16 2>&1 `
if [ $? -ne 0 ]; then
  echo "      (F) Failed to stop NT Service vwkernel and vwlogger"
  echo "          Error is:"
  echo "          $err_txt"
  echo "          Exiting..."
  exit 88
fi

# Stop DB2
echo "      (I) Stopping DB2"
err_txt=`$DB2CMD db2stop force 2>&1 `
if [ $? -ne 0 ]; then
  blah=`echo "$err_txt" | grep -i $sql_db_not_started `
  if [ $? -eq 0 ]; then
    :;			# If db not started that is OK
  else
    echo "      (F) Failed to stop DB2."
    echo "          Error is:"
    echo "          $err_txt"
    echo "          Exiting..."
    exit 89
  fi
fi


# Start DB2
echo "      (I) Starting DB2"
err_txt=`$DB2CMD db2start 2>&1 `
# Removed for defect 131629
#if [ $? -ne 0 ]; then
#  echo "      (F) Failed to start DB2."
#  echo "          Error is:"
#  echo "          $err_txt"
#  echo "          Exiting..."
#  exit 90
#fi

# Make the backup
# We can have multiple versions. The maintenance of backup versions will not be automated.
echo "      (I) Making Backup"
blah=`$DB2EXEC_LOG -v backup db "$TWH_CTRL_DB" to \'$CORE_BACKUP_DIR\' `
if [ $? -ne 0 ]; then
  echo "      (F) Failure detected during backup"
  echo "          for Database ($TWH_CTRL_DB)."
  echo "          Check messages in DB2 log ($DB2_INSTALL_LOG)."
  echo "          Exiting..."
  exit 90
fi


echo "      (I) Done making database backup."
return 0;
}


###


# Update the DWC Metadata db (normally tag file work)
update_control_server()
{
echo ""
echo "    Updating Control Server MD DB:"


# Run drop script
echo "      (I) Updating DWC MD schema"
if [ ! -r "$PATCH_GENERIC_DIR"/install/sql/twh_maintenance_tag_patch_1.sql ]; then
  echo "      (F) Could not locate the twh_maintenance_tag_patch_1.sql file."
  exit 91
fi


# Create a backup directory for the Core
# Create an app backup directory for DWC tag file work
if [ ! -d "$CORE_BACKUP_DIR" ]; then
  blah=`mkdir -p "$CORE_BACKUP_DIR" 2>&1`
  if [ $? -ne 0 ]; then
    echo "      (F) Unable to create application backup directory at"
    echo "          ($CORE_BACKUP_DIR)."
    exit 92
  fi

  # Set permissions
  blah=`chmod -R 755 "$CORE_BACKUP_DIR" `
fi


# Backup MD db for insurance
execute_md_db_backup;


# Now run the MD DB updates
gr_build_exec_script "$TWH_CTRL_DB" "$PATCH_GENERIC_DIR/install/sql" "twh_maintenance_tag_patch_1.sql" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CTRL_DB" $TRUE;


# Now run the MD DB updates for RPI
gr_build_exec_script "$TWH_CTRL_DB" "$PATCH_GENERIC_DIR/rpi/sql" "rpi_cs_patch2.sql" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CTRL_DB" $TRUE;


echo "      (I) Done updating the Control Server MD database"
return 0;
}


###


#
create_new_patch_directories()
{
echo "==> Adding Patch Directories"

# Add a directory for marker files to be kept
if [ ! -d "$TWH_TOPDIR"/.installed  ]; then
  blah=`mkdir -p "$TWH_TOPDIR"/.installed `
  if [ $? -ne 0 ]; then
    echo "      (F) Failed to create the (.installed) directory"
    exit 81
  fi
  echo "      (I) Created .installed directory"
else
  echo "      (I) .installed directory exists"
fi


# Add a directory for the patch logs to go
if [ ! -d "$PATCH_LOGS_DIR" ]; then
  blah=`mkdir -p "$PATCH_LOGS_DIR" `
  if [ $? -ne 0 ]; then
    echo "      (F) Failed to create the Fix Pack logs directory"
    exit 82
  fi
  echo "      (I) Created Fix Pack logs directory"
else
  echo "      (I) Fix Pack logs directory exists"
fi




# Synch RPI and Datamart dirs for CDW and Mart host
if [ ! -d "$TWH_TOPDIR/rpi/sql" ]; then
  blah=`mkdir -p "$TWH_TOPDIR/rpi/sql" `
  if [ $? -ne 0 ]; then
    echo "      (F) Failed to create the rpi/sql directory"
    exit 82
  fi
  echo "      (I) Created rpi/sql directory"
else
  echo "      (I) rpi/sql directory exists"
fi


#
if [ ! -d "$TWH_TOPDIR/rpi/schema" ]; then
  blah=`mkdir -p "$TWH_TOPDIR/rpi/schema" `
  if [ $? -ne 0 ]; then
    echo "      (F) Failed to create the rpi/schema directory"
    exit 82
  fi
  echo "      (I) Created rpi/schema directory"
else
  echo "      (I) rpi/schema directory exists"
fi


# Tmp place before they go to ps/cd
if [ ! -d "$TWH_TOPDIR/rpi/bin" ]; then
  blah=`mkdir -p "$TWH_TOPDIR/rpi/bin" `
  if [ $? -ne 0 ]; then
    echo "      (F) Error trying to create rpi/bin directory."
    exit 1
  fi
fi


# Bug fix... If a file called sql exists under install on Unix - remove it
if [ -f "$TWH_TOPDIR/install/sql" ]; then
  blah=`rm -f "$TWH_TOPDIR/install/sql" `
  echo "      (W) Found SQL file where SQL directory should be.  Removing..."
fi


# Create install/sql
if [ ! -d "$TWH_TOPDIR/install/sql" ]; then
  blah=`mkdir -p "$TWH_TOPDIR/install/sql" `
  if [ $? -ne 0 ]; then
    echo "      (F) Failed to create the install/sql directory"
    exit 82
  fi
  echo "      (I) Created install/sql directory"
else
  echo "      (I) install/sql directory exists"
fi


# Set permissions for whole directory
echo "      (I) Setting permissions"
blah=`chmod -R 755 "$TWH_TOPDIR"/rpi `

blah=`chmod -R 755 "$TWH_TOPDIR"/install `

blah=`chmod -R 755 "$TWH_TOPDIR"/.installed `

blah=`chmod -R 755 "$PATCH_LOGS_DIR" `


return 0;
}


###


# General routine to copy tools files over
update_tools()
{
echo ""
echo "    Updating Core tools:"


#############
# tools/bin #
#############
echo "      (I) Copying files to tools/bin directory"

# No execsql for Linux
if [ "$INTERP" != "LINUX" ]; then
  if [ ! -w "$TWH_TOPDIR"/tools/bin ]; then
    echo "      (F) Cannot write to ($TWH_TOPDIR/tools/bin) directory."
    exit 1
  fi


  # Overwrite the original
  cp -f "$PATCH_INTERP_DIR"/tools/bin/execsql*  "$TWH_TOPDIR"/tools/bin
  if [ $? -ne 0 ]; then
    echo "      (F) Error trying to copy execsql install media."
    exit 1
  fi
fi


cp -f "$PATCH_GENERIC_DIR"/tools/bin/sqlscript.sh  "$TWH_TOPDIR"/tools/bin
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy sqlscript.sh file."
  exit 1
fi


cp -f "$PATCH_GENERIC_DIR"/tools/bin/runReport.sh "$TWH_TOPDIR"/tools/bin
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy runReport.sh file."
  exit 1
fi


cp -f "$PATCH_GENERIC_DIR"/tools/bin/rollup.sh  "$TWH_TOPDIR"/tools/bin
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy install media (rollup.sh)."
  exit 1
fi

 
cp -f "$PATCH_GENERIC_DIR"/tools/bin/rollup.db2.template  "$TWH_TOPDIR"/tools/bin
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy install media (rollup.db2.template)."
  exit 1
fi


# Jar files used by REE and canned report loading
# Delete old files
rm -f "$TWH_TOPDIR"/tools/bin/twh_ree_110.jar
rm -f "$TWH_TOPDIR"/tools/bin/twh_ree_1101.jar	# Internal e-fix

# Copy it in
cp -f "$PATCH_GENERIC_DIR"/tools/bin/twh_ree.jar "$TWH_TOPDIR"/tools/bin
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy twh_ree.jar file."
  exit 1
fi


# This jar Can never updated to 1.1.1 as it is called
# from static app installer
cp -f "$PATCH_GENERIC_DIR"/tools/bin/com.tivoli.twh.nls@1.1.0.jar "$TWH_TOPDIR"/tools/bin
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy install media."
  exit 1
fi


cp -f "$PATCH_GENERIC_DIR"/tools/bin/com.tivoli.twh.rpi.impl.jar  "$TWH_TOPDIR"/tools/bin
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy com.tivoli.twh.rpi.impl.jar file."
  exit 1
fi


cp -f "$PATCH_GENERIC_DIR"/tools/bin/com.tivoli.twh.rpi.repnav.impl.jar  "$TWH_TOPDIR"/tools/bin
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy com.tivoli.twh.rpi.repnav.impl.jar file."
  exit 1
fi


cp -f "$PATCH_GENERIC_DIR"/tools/bin/com.tivoli.twh.utilities.jar  "$TWH_TOPDIR"/tools/bin
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy com.tivoli.twh.rpi.utilities.jar file."
  exit 1
fi


cp -f "$PATCH_GENERIC_DIR"/tools/bin/twh_rpiselections_update.jar  "$TWH_TOPDIR"/tools/bin
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy twh_rpiselections_update.jar  file."
  exit 1
fi


###############
# install/sql #
###############
echo "      (I) Copying files to install/sql programs"

if [ ! -w "$TWH_TOPDIR"/install/sql ]; then
  echo "      (F) Cannot write to ($TWH_TOPDIR/install/sql) directory."
  exit 1
fi
  

cp -f "$PATCH_GENERIC_DIR"/install/sql/twh_dwc_delkeys.sql "$TWH_TOPDIR"/install/sql 
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy twh_dwc_delkeys.sql file."
  exit 1
fi

  
cp -f "$PATCH_GENERIC_DIR"/install/sql/list_deletions.sql "$TWH_TOPDIR"/install/sql 
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy list_deletions.sql file."
  exit 1
fi


cp -f "$PATCH_GENERIC_DIR"/install/sql/delete_data.sql "$TWH_TOPDIR"/install/sql 
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy delete_data.sql file."
  exit 1
fi


cp -f "$PATCH_GENERIC_DIR"/install/sql/list_deletions_step1a.sql "$TWH_TOPDIR"/install/sql 
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy list_deletions_step1a.sql."
  exit 1
fi


cp -f "$PATCH_GENERIC_DIR"/install/sql/list_obj.sql "$TWH_TOPDIR"/install/sql
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy list_obj.sql"
  exit 1
fi



cp -f "$PATCH_GENERIC_DIR"/install/sql/upddbm.sql "$TWH_TOPDIR"/install/sql
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy upddbm.sql."
  exit 1
fi


cp -f "$PATCH_GENERIC_DIR"/install/sql/updsel.sql "$TWH_TOPDIR"/install/sql
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy  updsel.sql."
  exit 1
fi


cp -f "$PATCH_GENERIC_DIR"/install/sql/clean.sql "$TWH_TOPDIR"/install/sql
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy  clean.sql."
  exit 1
fi


###############
# install/bin #
###############
echo "      (I) Copying files to install/bin"

cp -f "$PATCH_GENERIC_DIR"/install/bin/twh_app_deinstall.sh "$TWH_TOPDIR"/install/bin
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy twh_app_deinstall.sh."
  exit 1
fi


cp -f "$PATCH_GENERIC_DIR"/install/bin/twh_gen_routines.sh "$TWH_TOPDIR"/install/bin
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy twh_gen_routines.sh."
  exit 1
fi


# Copy in new WPack Patching tools
cp -f "$PATCH_GENERIC_DIR"/install/bin/tedw_wpack_patchadm.sh "$TWH_TOPDIR"/install/bin
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy tedw_wpack_patchadm.sh."
  exit 1
fi


cp -f "$PATCH_GENERIC_DIR"/install/bin/twh_create_wpack_cfg_file.sh "$TWH_TOPDIR"/install/bin
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy twh_create_wpack_cfg_file.sh."
  exit 1
fi


cp -f "$PATCH_GENERIC_DIR"/install/bin/twh_app_patcher.sh "$TWH_TOPDIR"/install/bin
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy twh_app_patcher.sh."
  exit 1
fi


cp -f "$PATCH_GENERIC_DIR"/install/bin/twh_core_after.sh "$TWH_TOPDIR"/install/bin
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy twh_core_after.sh."
  exit 1
fi


cp -f "$PATCH_GENERIC_DIR"/install/bin/twh_core_startup_after.sh "$TWH_TOPDIR"/install/bin
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy twh_core_startup_after.sh."
  exit 1
fi


# Put this file in
cp -f "$PATCH_GENERIC_DIR"/install/bin/twh_core_patcher.sh "$TWH_TOPDIR"/install/bin
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy twh_core_patcher.sh."
  exit 1
fi


###########
# rpi/sql #
###########
echo "      (I) Copying files to rpi/sql"

cp -f "$PATCH_GENERIC_DIR"/rpi/sql/rpi_export1_create_table.sql  "$TWH_TOPDIR"/rpi/sql
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy rpi_export1_create_table.sql."
  exit 1
fi


cp -f "$PATCH_GENERIC_DIR"/rpi/sql/rpi_export2_create_table.sql  "$TWH_TOPDIR"/rpi/sql
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy rpi_export2_create_table.sql."
  exit 1
fi


cp -f "$PATCH_GENERIC_DIR"/rpi/sql/rpi_export3_create_table.sql  "$TWH_TOPDIR"/rpi/sql
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy rpi_export3_create_table.sql."
  exit 1
fi


cp -f "$PATCH_GENERIC_DIR"/rpi/sql/rpi_export3_process.sql  "$TWH_TOPDIR"/rpi/sql
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy rpi_export3_process.sql."
  exit 1
fi


cp -f "$PATCH_GENERIC_DIR"/rpi/sql/rpi_droplist_tables.sql "$TWH_TOPDIR"/rpi/sql
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy rpi_droplist_tables.sql file."
  exit 1
fi


cp -f "$PATCH_GENERIC_DIR"/rpi/sql/rpi_listmetrics_create_table.sql "$TWH_TOPDIR"/rpi/sql
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy rpi_listmetrics_create_table.sql file."
  exit 1
fi


cp -f "$PATCH_GENERIC_DIR"/rpi/sql/rpi_listmetrics_process_table.sql  "$TWH_TOPDIR"/rpi/sql
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy rpi_list_metrics_process_table.sql file."
  exit 1
fi


cp -f "$PATCH_GENERIC_DIR"/rpi/sql/rpi_listreports_process_table.sql  "$TWH_TOPDIR"/rpi/sql
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy rpi_listreports_process_table.sql file."
  exit 1
fi


##############
# rpi/schema #
##############
echo "      (I) Copying files to rpi/schema"

if [ ! -w "$TWH_TOPDIR"/rpi/schema ]; then
  echo "      (F) Cannot move media to ($TWH_TOPDIR/rpi/schema) directory."
  exit 1
fi


cp -f "$PATCH_GENERIC_DIR"/rpi/schema/* "$TWH_TOPDIR"/rpi/schema
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy install media."
  exit 1
fi


############
# rpi/bin #
############
echo "      (I) Copying files to rpi/bin"

cp -f "$PATCH_GENERIC_DIR"/rpi/bin/com.tivoli.twh.rpi.impl@1.1.2.jar "$TWH_TOPDIR"/rpi/bin
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy com.tivoli.twh.rpi.impl@1.1.1.jar file."
  exit 1
fi


# Stays at 1.1.0
cp -f "$PATCH_GENERIC_DIR"/rpi/bin/com.tivoli.twh.rpi.nls@1.1.0.jar "$TWH_TOPDIR"/rpi/bin
if [ $? -ne 0 ]; then
  echo "      (F) Error trying to copy com.tivoli.twh.rpi.nls@1.1.0.jar file."
  exit 1
fi




# Fix defect 136383 - If not there we cannot fix
#
if [ -r "$TWH_TOPDIR"/install/bin/twh_app_deinstall.cfg ]; then
  cat "$TWH_TOPDIR"/install/bin/twh_app_deinstall.cfg | sed 's/means that c:\/temp should be/means that c:\\\temp should be/' > "$TMPDIR"/_twh_app_deinstall.cfg
  rm -f "$TWH_TOPDIR"/install/bin/twh_app_deinstall.cfg
  cp -f "$TMPDIR"/_twh_app_deinstall.cfg "$TWH_TOPDIR"/install/bin/twh_app_deinstall.cfg
  rm -f "$TMPDIR"/_twh_app_deinstall.cfg 
fi


# Set permissions topdown
echo "      (I) Setting permissions"
blah=`chmod -R 755 "$TWH_TOPDIR"/tools `

# Set permissions for whole directory
blah=`chmod -R 755 "$TWH_TOPDIR"/install `

# Set permissions for whole directory
blah=`chmod -R 755 "$TWH_TOPDIR"/rpi `


echo "      (I) Done updating the core tools"
return 0;
}


###

update_ps()
{
echo ""
echo "    Updating PS to patch 2.2.3 (plus efix 1):"


echo "      (I) Setting up exec path and run variables"
# Env setup
if [ $UNIX_INTERP -eq $FALSE ]; then
  echo "            Path used will be 'w32-ix86'"
  generic_jar_path=`echo "${PATCH_GENERIC_DIR}/ps/" | sed 's/\//\\\\/g'`
 

  # PS workaround for wcmd.bat bug
  wcmd_prog="_wcmd.bat"
  wcmd_prog_remove="$ps_exec_path_private/wcmd.bat"
  ps_win_path=`echo "$PS_HOME/bin/private/w32-ix86" | sed 's/\//\\\\/g'`
  PS_TDW_TEMPDIR=`echo "$PS_HOME/twhtemp/" | sed 's/\//\\\\/g'`
  wcmd_fix_file="@echo off
REM Runtime TEDW fix for PS wcmd.bat problem

call ${ps_win_path}\wcmd.bat "%*"
exit %ERRORLEVEL%
"	# End file content

  # Write tmp bat file fix
  echo "$wcmd_fix_file" > "$TMPDIR"/$wcmd_prog
  chmod 777 "$TMPDIR"/$wcmd_prog
else
  echo "            Path used will be 'generic_unix'"
  wcmd_prog="$ps_exec_path_private/wcmd.sh"
  wcmd_prog_remove="$ps_exec_path_private/wcmd.sh"
  generic_jar_path="${PATCH_GENERIC_DIR}/ps/"
  PS_TDW_TEMPDIR="$PS_HOME/twhtemp/"
fi


# Start server incase...
manage_ps_servers $START_PS;


# Remove old jars
echo "            Removing old jar files..."
# rpi.impl
blah=`$wcmd_prog_remove cds uninstall "com.tivoli.pfdk.ful.util@2.2.1.jar" 2>&1 `
blah=`$wcmd_prog_remove cds uninstall "com.tivoli.pfdk.ful.util@2.2.2.jar" 2>&1 `

# 
blah=`"$wcmd_prog" cds install "${generic_jar_path}com.tivoli.pfdk.scriptharness.util@2.2.1.jar" 2>&1 `
blah=`"$wcmd_prog" cds install "${generic_jar_path}com.tivoli.pfdk.scriptharness.util@2.2.2.jar" 2>&1 `

#
blah=`"$wcmd_prog" cds install "${generic_jar_path}com.tivoli.pf.pfconsole.impl@2.2.1.jar" 2>&1 `
blah=`"$wcmd_prog" cds install "${generic_jar_path}com.tivoli.pf.pfconsole.impl@2.2.2.jar" 2>&1 `

#
blah=`"$wcmd_prog" cds install "${generic_jar_path}PsCommonInt@2.2.1.jar" 2>&1 `
blah=`"$wcmd_prog" cds install "${generic_jar_path}PsCommonInt@2.2.2.jar" 2>&1 `

#
blah=`"$wcmd_prog" cds install "${generic_jar_path}icu4j@1.3.1.jar" 2>&1 `



if [ ! -w "$PS_HOME"/cd ]; then
  echo "      (F) Cannot write to ($PS_HOME/cd) directory."
  exit 116
fi


# Input into PS
echo "      (I) Inserting new Jar files into PS"

# Needed NT hack below
cur_dir=`pwd`
cd "$TMPDIR"

# First lets create a temp dir under the PS directory.  This is so the PS
# services are guaranteed to be able to read the packages as the are 
# installed.
if [ ! -d "$PS_TDW_TEMPDIR" ]; then
    blah=`mkdir -p "$PS_TDW_TEMPDIR"`
    if [ $? -ne 0 ]; then
        echo "      (F) Unable to create temporay PS directory."
        echo "          ($PS_TDW_TEMPDIR))."
        exit 92
    fi

    if [ ! -w "$PS_TDW_TEMPDIR" ]; then
        echo "      (F) Cannot write to ($PS_TDW_TEMPDIR) directory."
        exit 116
    fi
fi

# Insert Jar 1
echo "            Jar 1...(com.tivoli.pfdk.ful.util@2.2.3.jar)"
cp -f "${generic_jar_path}com.tivoli.pfdk.ful.util@2.2.3.jar" "${PS_TDW_TEMPDIR}com.tivoli.pfdk.ful.util@2.2.3.jar"
if [ $? -ne 0 ]; then
    echo "      (F) Error trying to copy com.tivoli.pfdk.ful.util@2.2.3.jar."
    exit 1
fi

blah=`"$wcmd_prog" cds install "${PS_TDW_TEMPDIR}com.tivoli.pfdk.ful.util@2.2.3.jar" 2>&1 `
rc=$?

if [ $rc -ne 0 -a $rc -ne 5 ]; then
  echo "      (F) Failed to insert jar file into PS using WCMD.  RC=($rc)"
  echo "          Jar was com.tivoli.pfdk.ful.util@2.2.3.jar"
  echo "          Error was:"
  echo "          $blah"
  exit 117
fi

# PS bug
if [ $rc -eq 5 ]; then
  echo "              (W) There may have been installation errors"
  echo "                  with this jar file.  Please review the"
  echo "                  stderr*.txt logs in the PS log/fwp_mcr"
  echo "                  directory."
fi


# Insert Jar 2
echo "            Jar 2...(com.tivoli.pfdk.scriptharness.util@2.2.3.jar)"
cp -f "${generic_jar_path}com.tivoli.pfdk.scriptharness.util@2.2.3.jar" "${PS_TDW_TEMPDIR}com.tivoli.pfdk.scriptharness.util@2.2.3.jar"
if [ $? -ne 0 ]; then
    echo "      (F) Error trying to copy com.tivoli.pfdk.scriptharness.util@2.2.3.jar."
    exit 1
fi

blah=`"$wcmd_prog" cds install "${PS_TDW_TEMPDIR}com.tivoli.pfdk.scriptharness.util@2.2.3.jar" 2>&1 `
rc=$?

if [ $rc -ne 0 -a $rc -ne 5 ]; then
  echo "      (F) Failed to insert jar file into PS using WCMD.  RC=($rc)"
  echo "         Jar was com.tivoli.pfdk.scriptharness.util@2.2.3.jar"
  echo "          Error was:"
  echo "          $blah"
  exit 118
fi

# PS bug
if [ $rc -eq 5 ]; then
  echo "              (W) There may have been installation errors"
  echo "                  with this jar file.  Please review the"
  echo "                  stderr*.txt logs in the PS log/fwp_mcr"
  echo "                  directory."
fi


# Insert Jar 3
echo "            Jar 3...(com.tivoli.pf.pfconsole.impl@2.2.3.jar)"
cp -f "${generic_jar_path}com.tivoli.pf.pfconsole.impl@2.2.3.jar" "${PS_TDW_TEMPDIR}com.tivoli.pf.pfconsole.impl@2.2.3.jar"
if [ $? -ne 0 ]; then
    echo "      (F) Error trying to copy com.tivoli.pf.pfconsole.impl@2.2.3.jar."
    exit 1
fi

blah=`"$wcmd_prog" cds install "${PS_TDW_TEMPDIR}com.tivoli.pf.pfconsole.impl@2.2.3.jar" 2>&1 `
rc=$?

if [ $rc -ne 0 -a $rc -ne 5 ]; then
  echo "      (F) Failed to insert jar file into PS using WCMD.  RC=($rc)"
  echo "         Jar was com.tivoli.pf.pfconsole.impl@2.2.3.jar"
  echo "          Error was:"
  echo "          $blah"
  exit 119
fi

# PS bug
if [ $rc -eq 5 ]; then
  echo "              (W) There may have been installation errors"
  echo "                  with this jar file.  Please review the"
  echo "                  stderr*.txt logs in the PS log/fwp_mcr"
  echo "                  directory."
fi


# Insert Jar 4
echo "            Jar 4...(PsCommonInt@2.2.3.jar)"
cp -f "${generic_jar_path}PsCommonInt@2.2.3.jar" "${PS_TDW_TEMPDIR}PsCommonInt@2.2.3.jar"
if [ $? -ne 0 ]; then
    echo "      (F) Error trying to copy PsCommonInt@2.2.3.jar."
    exit 1
fi

blah=`"$wcmd_prog" cds install "${PS_TDW_TEMPDIR}PsCommonInt@2.2.3.jar" 2>&1 `
rc=$?

if [ $rc -ne 0 -a $rc -ne 5 ]; then
  echo "      (F) Failed to insert jar file into PS using WCMD.  RC=($rc)"
  echo "         Jar was PsCommonInt@2.2.3.jar"
  echo "          Error was:"
  echo "          $blah"
  exit 120
fi

# PS bug
if [ $rc -eq 5 ]; then
  echo "              (W) There may have been installation errors"
  echo "                  with this jar file.  Please review the"
  echo "                  stderr*.txt logs in the PS log/fwp_mcr"
  echo "                  directory."
fi


# File from PS 2.2.3 efix
# Insert Jar 5
echo "            Jar 5...(icu4j@1.3.1.jar)"

# First have to uninstall
echo "              Removing old jar file..."
blah=`$wcmd_prog_remove cds uninstall "icu4j@1.3.1.jar" 2>&1 `


# Now install
echo "              Inserting jar file..."
cp -f "${generic_jar_path}icu4j@1.3.1.jar" "${PS_TDW_TEMPDIR}icu4j@1.3.1.jar"
if [ $? -ne 0 ]; then
    echo "      (F) Error trying to copy icu4j@1.3.1.jar."
    exit 1
fi

blah=`"$wcmd_prog" cds install "${PS_TDW_TEMPDIR}icu4j@1.3.1.jar" 2>&1 `
rc=$?

if [ $rc -ne 0 -a $rc -ne 5 ]; then
  echo "      (F) Failed to insert jar file into PS using WCMD.  RC=($rc)"
  echo "         Jar was icu4j@1.3.1.jar"
  echo "          Error was:"
  echo "          $blah"
  exit 120
fi

# PS bug
if [ $rc -eq 5 ]; then
  echo "              (W) There may have been installation errors"
  echo "                  with this jar file.  Please review the"
  echo "                  stderr*.txt logs in the PS log/fwp_mcr"
  echo "                  directory."
fi


# Go back to current
cd "$cur_dir"


# Don't need to recycle the servers here as we need to install RPI
# Recycle the servers
# Stop servers
#echo "      (I) Stopping MCR and WC servers to Recycle servers"
#manage_ps_servers $RECYCLE_PS;


# Cleanup
if [ $UNIX_INTERP -eq $FALSE ]; then
  rm -f "$TMPDIR"/$wcmd_prog
fi

# remove temp directory
rm -fr "${PS_TDW_TEMPDIR}"


echo "      (I) Done updating PS"
return 0;
}


###


# Update the jar files in PS for RPI
update_rpi()
{
echo ""
echo "    Updating RPI environment:"


echo "      (I) Setting up exec path and run variables"
# Env setup
if [ $UNIX_INTERP -eq $FALSE ]; then
  echo "            Path used will be 'w32-ix86'"
  generic_jar_path=`echo "${PATCH_GENERIC_DIR}/rpi/bin/" | sed 's/\//\\\\/g'`
 

  # PS workaround for wcmd.bat bug
  wcmd_prog="_wcmd.bat"
  wcmd_prog_remove="$ps_exec_path_private/wcmd.bat"
  ps_win_path=`echo "$PS_HOME/bin/private/w32-ix86" | sed 's/\//\\\\/g'`
  PS_TDW_TEMPDIR=`echo "$PS_HOME/twhtemp/" | sed 's/\//\\\\/g'`
  wcmd_fix_file="@echo off
REM Runtime TEDW fix for PS wcmd.bat problem

call ${ps_win_path}\wcmd.bat "%*"
exit %ERRORLEVEL%
"	# End file content

  # Write tmp bat file fix
  echo "$wcmd_fix_file" > "$TMPDIR"/$wcmd_prog
  chmod 777 "$TMPDIR"/$wcmd_prog
else
  echo "            Path used will be 'generic_unix'"
  wcmd_prog="$ps_exec_path_private/wcmd.sh"
  wcmd_prog_remove="$ps_exec_path_private/wcmd.sh"
  generic_jar_path="${PATCH_GENERIC_DIR}/rpi/bin/"
  PS_TDW_TEMPDIR="$PS_HOME/twhtemp/"
fi


# Note PS servers should already be running because of patch 2.2.2
# Remove old jars
echo "            Removing old jar files..."
# rpi.impl
blah=`$wcmd_prog_remove cds uninstall "com.tivoli.twh.rpi.impl@1.1.0.jar" 2>&1 `
blah=`$wcmd_prog_remove cds uninstall "com.tivoli.twh.rpi.impl@1.1.01.jar" 2>&1 `
blah=`$wcmd_prog_remove cds uninstall "com.tivoli.twh.rpi.impl@1.1.1.jar" 2>&1 `

# rpi.nls
blah=`$wcmd_prog_remove cds uninstall "com.tivoli.twh.rpi.nls@1.1.0.jar" 2>&1 `

# rpi.security
blah=`$wcmd_prog_remove cds uninstall "com.tivoli.twh.rpi.security.impl@1.1.0.jar" 2>&1 `
blah=`$wcmd_prog_remove cds uninstall "com.tivoli.twh.rpi.security.impl@1.1.1.jar" 2>&1 `

# rpi.utilities
blah=`$wcmd_prog_remove cds uninstall "com.tivoli.twh.utilities@1.1.0.jar" 2>&1 `
blah=`$wcmd_prog_remove cds uninstall "com.tivoli.twh.utilities@1.1.1.jar" 2>&1 `



# Input into PS
echo "      (I) Inserting new RPI Jar files into PS"

# Needed NT hack below
cur_dir=`pwd`
cd "$TMPDIR"

# First lets create a temp dir under the PS directory.  This is so the PS
# services are guaranteed to be able to read the packages as the are 
# installed.
if [ ! -d "${PS_TDW_TEMPDIR}" ]; then
    blah=`mkdir -p "${PS_TDW_TEMPDIR}"`
    if [ $? -ne 0 ]; then
        echo "      (F) Unable to create temporay PS directory."
        echo "          (${PS_TDW_TEMPDIR}))."
        exit 92
    fi

    if [ ! -w "${PS_TDW_TEMPDIR}" ]; then
        echo "      (F) Cannot write to (${PS_TDW_TEMPDIR}) directory."
        exit 116
    fi
fi

# Insert Jar 1
echo "            Jar 1...(com.tivoli.twh.rpi.impl@1.1.2.jar)"
cp -f "${generic_jar_path}com.tivoli.twh.rpi.impl@1.1.2.jar" "${PS_TDW_TEMPDIR}com.tivoli.twh.rpi.impl@1.1.2.jar"
if [ $? -ne 0 ]; then
    echo "      (F) Error trying to copy com.tivoli.twh.rpi.impl@1.1.2.jar."
    exit 1
fi

blah=`$wcmd_prog cds install "${PS_TDW_TEMPDIR}com.tivoli.twh.rpi.impl@1.1.2.jar" 2>&1 `
rc=$?

if [ $rc -ne 0 -a $rc -ne 5 ]; then
  echo "      (F) Failed to insert jar file into PS using WCMD.  RC=($rc)"
  echo "          Jar was com.tivoli.twh.rpi.impl@1.1.2.jar"
  echo "          Error was:"
  echo "          $blah"
  exit 121
fi

# PS bug
if [ $rc -eq 5 ]; then
  echo "              (W) There may have been installation errors"
  echo "                  with this jar file.  Please review the"
  echo "                  stderr*.txt logs in the PS log/fwp_mcr"
  echo "                  directory."
fi


# Insert Jar 2
echo "            Jar 2...(com.tivoli.twh.rpi.nls@1.1.0.jar)"
cp -f "${generic_jar_path}com.tivoli.twh.rpi.nls@1.1.0.jar" "${PS_TDW_TEMPDIR}com.tivoli.twh.rpi.nls@1.1.0.jar"
if [ $? -ne 0 ]; then
    echo "      (F) Error trying to copy com.tivoli.twh.rpi.nls@1.1.0.jar."
    exit 1
fi

blah=`$wcmd_prog cds install "${PS_TDW_TEMPDIR}com.tivoli.twh.rpi.nls@1.1.0.jar" 2>&1 `
rc=$?

if [ $rc -ne 0 -a $rc -ne 5 ]; then
  echo "      (F) Failed to insert jar file into PS using WCMD.  RC=($rc)"
  echo "          Jar was com.tivoli.twh.rpi.nls@1.1.0.jar"
  echo "          Error was:"
  echo "          $blah"
  exit 121
fi

# PS bug
if [ $rc -eq 5 ]; then
  echo "              (W) There may have been installation errors"
  echo "                  with this jar file.  Please review the"
  echo "                  stderr*.txt logs in the PS log/fwp_mcr"
  echo "                  directory."
fi


# Insert Jar 3
echo "            Jar 3...(com.tivoli.twh.rpi.security.impl@1.1.2.jar)"
cp -f "${generic_jar_path}com.tivoli.twh.rpi.security.impl@1.1.2.jar" "${PS_TDW_TEMPDIR}com.tivoli.twh.rpi.security.impl@1.1.2.jar"
if [ $? -ne 0 ]; then
    echo "      (F) Error trying to copy com.tivoli.twh.rpi.security.impl@1.1.2.jar."
    exit 1
fi

blah=`$wcmd_prog cds install "${PS_TDW_TEMPDIR}com.tivoli.twh.rpi.security.impl@1.1.2.jar" 2>&1 `
rc=$?

if [ $rc -ne 0 -a $rc -ne 5 ]; then
  echo "      (F) Failed to insert jar file into PS using WCMD.  RC=($rc)"
  echo "          Jar was com.tivoli.twh.rpi.security.impl@1.1.2.jar"
  echo "          Error was:"
  echo "          $blah"
  exit 121
fi

# PS bug
if [ $rc -eq 5 ]; then
  echo "              (W) There may have been installation errors"
  echo "                  with this jar file.  Please review the"
  echo "                  stderr*.txt logs in the PS log/fwp_mcr"
  echo "                  directory."
fi


# Insert Jar 4
echo "            Jar 4...(com.tivoli.twh.utilities@1.1.2.jar)"
cp -f "${generic_jar_path}com.tivoli.twh.utilities@1.1.2.jar" "${PS_TDW_TEMPDIR}com.tivoli.twh.utilities@1.1.2.jar"
if [ $? -ne 0 ]; then
    echo "      (F) Error trying to copy com.tivoli.twh.utilities@1.1.2.jar."
    exit 1
fi

blah=`$wcmd_prog cds install "${PS_TDW_TEMPDIR}com.tivoli.twh.utilities@1.1.2.jar" 2>&1 `
rc=$?

if [ $rc -ne 0 -a $rc -ne 5 ]; then
  echo "      (F) Failed to insert jar file into PS using WCMD.  RC=($rc)"
  echo "          Jar was com.tivoli.twh.utilities@1.1.2.jar"
  echo "          Error was:"
  echo "          $blah"
  exit 121
fi

# PS bug
if [ $rc -eq 5 ]; then
  echo "              (W) There may have been installation errors"
  echo "                  with this jar file.  Please review the"
  echo "                  stderr*.txt logs in the PS log/fwp_mcr"
  echo "                  directory."
fi


# Go back to current
cd "$cur_dir"


# Recycle the servers
# Stop servers
echo "      (I) Stopping MCR and WC servers to Recycle servers"
manage_ps_servers $RECYCLE_PS;


# Cleanup
if [ $UNIX_INTERP -eq $FALSE ]; then
  rm -f "$TMPDIR"/$wcmd_prog
fi

# remove temp directory
rm -fr "${PS_TDW_TEMPDIR}"

echo "      (I) Done updating RPI PS files"
return 0;
}


###

#
update_db2jdbc()
{
echo ""
echo "    Updating db2java.zip file:"


"$PATCH_GENERIC_DIR"/install/bin/conv_db2_jdbc.sh "$TWH_TOPDIR"/tools/bin "$DB2_INST_DIR"
if [ $? -ne 0 ]; then
  echo "      (F) Error during update of db2java.zip file in $TWH_TOPDIR/tools/bin directory"
  exit 127
fi

if [ $RPI_CHECKED -eq $TRUE ]; then
  manage_ps_servers $STOP_PS
  
  "$PATCH_GENERIC_DIR"/install/bin/conv_db2_jdbc.sh "$PS_HOME"/db2 "$DB2_INST_DIR"
  if [ $? -ne 0 ]; then
    echo "      (F) Error during update of db2java.zip file in $TWH_TOPDIR/db2 directory"
    exit 128
  fi
  
  manage_ps_servers $START_PS
fi

echo "      (I) Done updating db2java.zip"
}


###


## Now update rpi.ss_cols db view
update_rpi_schema()
{
echo ""
echo "    Updating RPI Schema environment:"

rpi_triggers_script="twh_rpi_triggers.sql"
rpi_drop_triggers_script="twh_rpi_drop_triggers.sql"


echo "      (I) Updating RPI schema view"
# Run schema update
if [ ! -r "$TWH_TOPDIR"/rpi/schema/twh_rpi_schema_update.sql ]; then
  echo "      (F) Could not execute the twh_rpi_schema_update.sql file."
  exit 131
fi

gr_build_exec_script "$TWH_CTRL_DB" "$TWH_TOPDIR/rpi/schema" "twh_rpi_schema_update.sql" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CTRL_DB" $TRUE;


# Drop and recreate triggers
echo "      (I) Updating RPI triggers"
gr_build_exec_script "$TWH_CTRL_DB" "$TWH_TOPDIR/rpi/schema" "$rpi_drop_triggers_script" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CTRL_DB" $FALSE;


# Create Triggers
gr_build_exec_script "$TWH_CTRL_DB" "$TWH_TOPDIR/rpi/schema" "$rpi_triggers_script" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT "#";
gr_exec_sql_script "$TWH_CTRL_DB" $TRUE "#";


echo "      (I) Done updating RPI Schema"
return 0;
}


clean_pwds()
{
# Remove the connect strings from any remaining files
ls "$TWH_TOPDIR"/install/bin/twh*install.cfg 2> /dev/null > "$TMPDIR"/cfgfiles.txt;
ls "$TWH_TOPDIR"/install/bin/twh*patcher.cfg 2> /dev/null >> "$TMPDIR"/cfgfiles.txt;

while :
  do
    read cfg_file
    if [ "$cfg_file" = "" ]; then
        break
    fi

    fname=`basename "$cfg_file" `
    cat "$cfg_file" | sed "s/DB2PASS=.*/DB2PASS=/" > "$TMPDIR"/clean_$fname
    rm -f "$cfg_file"
    mv "$TMPDIR"/clean_$fname "$TWH_TOPDIR"/install/bin/$fname

  done < "$TMPDIR"/cfgfiles.txt
rm -f "$TMPDIR"/cfgfiles.txt


return 0;
}


###


# Start and stops PS servers
manage_ps_servers()
{
ps_action=$1


# Env setup
if [ $UNIX_INTERP -eq $FALSE ]; then
  ps_exec_path="$PS_HOME/bin/w32-ix86"
  ps_exec_path_private="$PS_HOME/bin/private/w32-ix86"
  extension=".bat"
  generic_jar_path=`echo "${PATCH_GENERIC_DIR}/ps/" | sed 's/\//\\\\/g'`
else
  echo "            Path used will be 'generic_unix'"
  ps_exec_path="$PS_HOME/bin/generic_unix"
  ps_exec_path_private="$PS_HOME/bin/private/generic_unix"
  extension=".sh"
  wcmd_prog="$ps_exec_path_private/wcmd.sh"
  generic_jar_path="${PATCH_GENERIC_DIR}/ps/"
fi


if [ ! -d "$PS_HOME" ]; then 
  echo "      (F) Could not locate PS home directory ($PS_HOME)"
  exit 132
fi


# Execute requested action
if [ $ps_action -eq $STOP_PS -o $ps_action -eq $RECYCLE_PS ]; then
  # Stop servers
  echo "      (I) Stopping MCR and WC servers"
  blah=`"$ps_exec_path_private"/stopwc${extension} 2>&1 `
  blah=`"$ps_exec_path_private"/stopmcr${extension} 2>&1 `
fi
  
# sleep between action
sleep 10


if [ $ps_action -eq $START_PS -o $ps_action -eq $RECYCLE_PS ]; then
  echo "      (I) Starting MCR server"
  blah=`"$ps_exec_path"/mcr${extension} 2>&1 `
  
  # Wait for them to start...
  echo "      (I) Waiting for MCR server to start"
  blah=`"$ps_exec_path_private"/waitmcr${extension} 2>&1 `
  rc=$?
  if [ $rc -ne 0 ]; then
    echo "      (F) Failed to run waitmcr program.  RC=($rc)"
    echo "          Error was:"
    echo "          $blah"
    exit 133
  fi
  
  echo "      (I) Starting WC server"
  blah=`"$ps_exec_path"/wc${extension} 2>&1 `
fi


return 0;
}

create_props()
{
NTTOPDIR=`echo "$TWH_TOPDIR" | sed 's/\//\\\\/g'`
CLASSPATH="$NTTOPDIR\tools\bin\IBMJCEfw.jar;$NTTOPDIR\tools\bin\IBMJCEProvider.jar;$NTTOPDIR\tools\bin\local_policy.jar;$NTTOPDIR\tools\bin\US_export_policy.jar;$NTTOPDIR\tools\bin\com.tivoli.twh.utilities.jar"
export CLASSPATH

JAVA="$TWH_TOPDIR/_jvm/bin/java.exe"

CMDARGS="RHpasswordVJ databasedriver COM.ibm.db2.jdbc.app.DB2Driver url jdbc:db2:TWH_MD userid $COPT_CTRL_DB2USER password $COPT_CTRL_DB2PASS"
                                                           
blah=`"$JAVA" com.tivoli.twh.utilities.VaultCmdLine put \\\"$NTTOPDIR\rpi\\\" .rpidb.properties $CMDARGS`
rc=$?

if [ $rc -ne 0 ]; then
  echo "      (F) Failed to create properties files.  RC=($rc)"
  exit 1;
fi

if [ ! -r "$TWH_TOPDIR/rpi/.rpidb.properties" ]; then
  echo "      (F) Failed to create properties files.  RC=($rc)"
  exit 1;
fi

return 0
}


###


# Tech Notes:
#	We use the NT DWC host as the anchor for all DB work for all
#	DB's (i.e. MD, CDW, and Mart).  So the other custom options would
#	be physical objects like copying files that must be done locally.
execute_install_driver()
{
echo "==> Executing Core Fix Pack Driver"


# Section for all pre-component options
read_core_comp_list_file;
validate_db2_level 7 1 60;
validate_patch_installable;
rm -f "$TMPDIR/$DB2_INSTALL_LOG"		# Remove old DB2 install log
gr_test_db2exec;
gr_mark_db2_log_for_install "MARKCP2";	# Core patch
create_new_patch_directories;


# Now install each patch component - this is in order
if [ $ADMIN_CHECKED -eq $TRUE ]; then
  # For this option we will do all the DB work for all components
  echo ""
  echo "==> Updating Admin component";

  # NT only for this!
  gr_check_interp_by_class $CLASS_WIN;

  update_control_server;
  update_tools;
  update_rpi_schema;
  update_db2jdbc;
  create_props;
fi


# CDW - Remember this is a local host function
if [ $CDW_CHECKED -eq $TRUE ]; then
  echo ""
  echo "==> Updating local CDW component";

  # Check acceptable interps
  gr_check_interp_by_class $CLASS_TYPE_1;


  if [ "$ADMIN_CHECKED" -ne $TRUE ]; then
    update_tools;
    update_db2jdbc;
  fi
fi


# Mart DB
if [ $MART_CHECKED -eq $TRUE ]; then
  echo ""
  echo "==> Updating local Mart component";

  # Check acceptable interps 
  gr_check_interp_by_class $CLASS_TYPE_1;

  if [ "$ADMIN_CHECKED" -ne $TRUE -a "$CDW_CHECKED" -ne $TRUE ]; then
    update_tools;
    update_db2jdbc;
  fi
fi


# RPI
if [ $RPI_CHECKED -eq $TRUE ]; then
  echo ""
  echo "==> Updating local RPI component";

  # Check acceptable interps
  gr_check_interp_by_class $CLASS_TYPE_2;

  if [ "$ADMIN_CHECKED" -ne $TRUE -a "$CDW_CHECKED" -ne $TRUE -a "$MART_CHECKED" -ne $TRUE  ]; then
    update_tools;
    update_db2jdbc;
  fi

  update_ps;
  update_rpi;
fi
echo ""


# Section for all post-component options
clean_pwds;


return 0;
}


###


# List the installed products and patches
#
exec_list_installed()
{
tmp_marker_file="$TMPDIR"/tmp_marker_list.txt
prod_count=0
prod_flag=0
date_val="01/01/2000"

echo "==> Listing the installed Product, Fix Pack, and E-Fix versions"
echo ""


# Get the 1.1.0 product marker
rm -f "$tmp_marker_file"
cur_dir=`pwd`
cd "$TWH_TOPDIR"
ls -l .twh_core_v1.1.0.marker 2> /dev/null > "$tmp_marker_file";

# Get any other product markers under new .installed dir
if [ -d "$TWH_TOPDIR"/.installed ]; then
  cd "$TWH_TOPDIR"/.installed
  ls -l .twh_core_v*.*.*.marker 2> /dev/null >> "$tmp_marker_file";
  cd "$cur_dir"
fi

echo ""
echo "    You have the following Product versions installed:"
echo ""
echo "          ==============="
echo ""


# Are any installed
if [ ! -s "$tmp_marker_file" ]; then
  echo "      (F) No Core installation versions detected on this system"
  echo "          This may reflect a failed Core installation attempt."
  echo "          Exiting..."
  echo ""
  exit 134
fi


# read the file to grab the versions
while :
  do
    read list_rec
    if [ "$list_rec" = "" ]; then
        break
    fi

    v_file=`echo $list_rec | awk '{print $9}' `
    file_date=`echo $list_rec | awk '{print $6,$7,$8}' `

    # Fetch the version info
    get_version_info "$v_file";


    # Print out list
    if [ "$HAVE_EFIX" = "$TRUE" ]; then
      echo "             Efix:         $TWH_EFIX_VER  installed on $file_date"
      continue
    fi

    if [ $TWH_PATCH_VER -eq 0 ]; then
      if [ $prod_flag -eq 1 ]; then
        echo "";
        prod_flag=0
      fi
      echo "          Product:     $TWH_VERSION_VAL  installed on $file_date"
      prod_flag=1
	prod_count=`expr $prod_count + 1 `		# bump count
    else
      echo "            Fix Pack:  $TWH_VERSION_VAL  installed on $file_date"
    fi


  done < "$tmp_marker_file"
#rm -f "$tmp_marker_file"


if [ $prod_count -eq 0 ]; then
  echo "          *  <None found>"
fi

echo ""
echo "          ==============="
echo ""
echo ""


echo "      (I) Done listing installed versions"
echo "          Exiting..."
exit 0

return 0;
}


###


##########################################################
# Get the release, maintenence release, and patch versions
# 	$1 = the version marker file
#	Output:
#		TWH_VERSION_VAL = The entire version
#		TWH_RELEASE_VER = The release digit
#		TWH_MAINT_VER = The maintenence digit
#		TWH_PATCH_VER = The patch digit
#
# Core versions will be
#	.twh_core_v1.2.3.marker
#                r.m.p
#     Where
#	r.	= Release digit
#	.m	= Maintenance digit
#	.p	= Patch digit
#
# Efix version format will be:
#	.twh_core_v1.1.1@efix_4.marker
#                          _e
#	Where
#	_e	= Efix number
##########################################################
get_version_info()
{
HAVE_EFIX="$FALSE"
marker_file="$1"
TWH_RELEASE_VER=0
TWH_MAINT_VER=0
TWH_PATCH_VER=0
TWH_EFIX_VER=0

if [ -z "$marker_file" ]; then
  echo "      (F) Marker file not supplied to function get_version_info"
  exit 136
fi

# get the release, maintenance release, and patch numbers
tmp_version_val=`echo "$marker_file" | sed "s/\.twh_core_v//" `
tmp_version_val=`echo "$tmp_version_val" | sed "s/\.marker//" `

# strip possible efix val
TWH_VERSION_VAL=` echo "$tmp_version_val" | sed "s/@efix_.*//" `


# Release
TWH_RELEASE_VER=`echo "$TWH_VERSION_VAL" | sed "s/\..*//g" `

# Maintenance version
TWH_MAINT_VER=`echo "$TWH_VERSION_VAL" |  sed "s/^[0-9]*//" | sed "s/[0-9]*$//" |  sed "s/\.//g" `

# Patch version
TWH_PATCH_VER=`echo "$TWH_VERSION_VAL" | sed "s/.*\.//g" `

# Efix version
blah=` echo "$tmp_version_val" | grep -i "@efix" `
if [ $? -eq 0 ]; then
  TWH_EFIX_VER=` echo "$tmp_version_val" | sed "s/.*@efix//" `
  HAVE_EFIX="$TRUE"
fi


return 0;
}


###


# 	$1 = Interger value for the component type
get_node_alias()
{
alias_type=$1
GR_NODE_ALIAS="NULL"


if [ -z "$alias_type" ]; then
  echo "      (F) No supplied value for alias_type in get_node_alias"
  exit 137
fi

# Set the requested node alias
if [ $alias_type -eq $CTRL_ALIAS_TYPE ]; then
  if [ -z "$TWH_CTRL_NODE_ALIAS" ]; then
    # Read the config file (persists the majic node number)
    write_core_comp_list_conf_file "TWH_CUSTOM" $alias_type;
  fi

  GR_NODE_ALIAS="$TWH_CTRL_NODE_ALIAS"
elif [ $alias_type -eq $CDW_ALIAS_TYPE ]; then
  if [ -z "$TWH_CDW_NODE_ALIAS" ]; then
    # Read the config file (persists the majic node number)
    write_core_comp_list_conf_file "TWH_CUSTOM" $alias_type;
  fi

  GR_NODE_ALIAS="$TWH_CDW_NODE_ALIAS"
elif [ $alias_type -eq $MART_ALIAS_TYPE ]; then
  if [ -z "$TWH_MART_NODE_ALIAS" ]; then
    # Read the config file (persists the majic node number)
    write_core_comp_list_conf_file "TWH_CUSTOM" $alias_type;
  fi

  GR_NODE_ALIAS="$TWH_MART_NODE_ALIAS"
fi


# make it public
export GR_NODE_ALIAS

echo "      (I) Node alias is $GR_NODE_ALIAS"
return 0;
}


###


# Clean up
clean_up()
{
tmp_conf_file="tmp_${TWH_CONFIG_FILE_NAME}"
move_conf_path="$PATCH_LOGS_DIR"

echo "==> Starting post-installation cleanup"

# Remove the password from the config file
echo "      (I) Cleaning passwords from Config file"
cat "$TWH_CONFIG_FILE" | sed "s/DB2PASS=.*/DB2PASS=/" > "$TMPDIR"/$tmp_conf_file
echo "      (I) Moving config file to $move_conf_path"
mv "$TMPDIR"/$tmp_conf_file "$move_conf_path"/${TWH_CONFIG_FILE_NAME}.done
if [ $UNIX_INTERP -eq $FALSE ]; then
  exec < "$move_conf_path"/$TWH_CONFIG_FILE_NAME.done
  # do this for NT because of file locking
fi
rm -f "$TWH_CONFIG_FILE"


# make a copy of the Core Patch install DB2 log for support
cp -f "$TMPDIR/$DB2_INSTALL_LOG" "$move_conf_path/$DB2_INSTALL_LOG@patch_v${TWH_VERSION}"
rm -f "$TMPDIR/$DB2_INSTALL_LOG"

# Safety net - Remove the connect strings from any remaining files
echo "      (I) Cleaning any remaining tmp sql files"
ls "$TMPDIR"/tmp_twh*.sql 2> /dev/null > "$TMPDIR"/sqlfiles.txt;
while :
  do
    read sql_file
    if [ "$sql_file" = "" ]; then
        break
    fi

    gr_clean_conn_string "$sql_file" "$TMPDIR";

  done < "$TMPDIR"/sqlfiles.txt
rm -f "$TMPDIR"/sqlfiles.txt


echo "      (I) Post-installation cleanup completed"
return 0;
}


###


#
post_details()
{
echo ""
echo "==> TEDW Core Fix Pack Install details:"
echo ""

# Hardcode for fp 1
echo "       TEDW Fix Pack Version Installed:           2"
echo "       Components installed on this host:"


if [ $ADMIN_CHECKED -eq $TRUE ]; then
  echo "         * ADMIN component local to this host"
fi
if [ $CDW_CHECKED -eq $TRUE ]; then
  echo "         * CDW component local to this host"
fi
if [ $MART_CHECKED -eq $TRUE ]; then
  echo "         * MART component local to this host"
fi
if [ $RPI_CHECKED -eq $TRUE ]; then
  echo "         * RPI component local to this host"
fi
echo ""


echo "       TEDW Core tools installed at:    $TWH_TOPDIR"
echo "       Additional logs located:         $PATCH_LOGS_DIR"
echo "       Fix Pack install log:            twh_install_v${TWH_VERSION}.log"
echo "       TEDW DB2 SQL Fix Pack log:       $DB2_INSTALL_LOG@patch_v${TWH_VERSION}"
if [ -n "$TWH_KEEP_TMP_FILES_TRUE" ]; then
  echo "       <Keeping tmp files>"
fi
echo ""
echo "       (I) TEDW Core Install details completed"
return 0;
}


###


#
prep_exit()
{
if [ ! -r "$FP1_CORE_PATCH_VERSION_FILE" ]; then
  touch "$FP1_CORE_PATCH_VERSION_FILE"
fi

# tag file to say we have a valid install.  Prevents reinstall
# Need to copy in the build info
if [ -s "$PATCH_GENERIC_DIR"/.build ]; then
  cat "$PATCH_GENERIC_DIR"/.build > "$CORE_PATCH_VERSION_FILE"
else
  echo "      (W) Could not locate build file in Fix Pack package."
fi

# Put the backward compatible (1.1 installer) file down
# Need to put down each FP level
touch "$BCOMPAT_CORE_PATCH_VERSION_FILE"
if [ ! -r "$FP1_BCOMPAT_CORE_PATCH_VERSION_FILE" ]; then
  touch "$FP1_BCOMPAT_CORE_PATCH_VERSION_FILE";
fi


echo ""
echo "==> TEDW Fix Pack Installation Successfully Completed!!!"


# Copy the tee'd log file of this install to our log directory
if [ $UNIX_INTERP -eq $FALSE ]; then
  INST_WIN_LOGDIR=`echo "$INST_WIN_LOGDIR" | sed 's/\\\\/\//g' `	# set Windows path
  cp -f "$INST_WIN_LOGDIR"/twh_install_v{$TWH_VERSION}.log "$PATCH_LOGS_DIR"/twh_install_v${TWH_VERSION}.log
else
  cp -f "$INST_UNIX_LOGDIR"/twh_install_v$TWH_VERSION.log "$PATCH_LOGS_DIR"/twh_install_v$TWH_VERSION.log
fi


# Turn off the Core context for the Generic routines
unset TWH_CORE_PATCH_CONTEXT


return 0;
}


###############################################################
############## Driver Code Section ############################
###############################################################

### Set Program Vars
# Non configurable vars 
# Required in all files using Generic Routines (gr_)
ARGS_RUN="$@"
ARGS_IN=$*
ARGS_NO=$#
PROGRAM=$0
gen_file_and_path=`echo $0 | sed 's/\\\\/\//g' `	# NT fix
PROGNAME=`basename "$gen_file_and_path"`
TRUE=1
FALSE=0
CUSTOM_INSTALL=$FALSE
ADMIN_CHECKED=$FALSE
RPI_CHECKED=$FALSE
CDW_CHECKED=$FALSE
MART_CHECKED=$FALSE


############
### Main ###
############

# Bring in the generic routines
include_gen_routines;


# If we have a path from the setup program display that as the name
if [ "$PROGRAM" = "generic\install\bin\twh_core_patcher.sh" ]; then
  if [ -n "$INST_WIN_LOGDIR" ]; then
    PROG_NAME="setup.bat"
    CFG_FILE_LOC="$INST_WIN_LOGDIR"
  else
    PROG_NAME="setup_unix.sh"
    CFG_FILE_LOC="$INST_UNIX_LOGDIR"
  fi
  LOCALLY_CALLED=$FALSE
else
  # Local prog name invoked
  PROG_NAME="twh_core_patcher.sh"
  LOCALLY_CALLED=$TRUE
fi


# We first need to process all arguments passed to the script.
# Have to watch out for spaces in the path
args_found=0
conf_val_found=$FALSE


while [ $# -ne 0 ];
  do
  # Now look for the command line option
  case $1 in

	# Config file
        --config|--CONFIG|--Config|-C|-c)
	      # Set flag for value found
	      conf_val_found=$TRUE
	      shift
	      TWH_CONFIG_FILE=`echo $1 | sed 's/\\\\/\//g'`
		if [ ! -r "$TWH_CONFIG_FILE" ]; then
		  echo "      (F) CDWIC0081E Value not valid for config file name ($TWH_CONFIG_FILE)"
		  exit 138
		fi
	      shift
    	      args_found=`expr $args_found + 1`
	      continue
            ;;

	# Version Info
        --version|--VERSION|--Version|-V|-v)
	      echo ""
            echo "  $PROG_NAME"
	      echo "  Version 1.1.2"
	      echo "  A Tivoli Enterprise Data Warehouse utility"
	      echo "  IBM Software"
	      echo ""
            exit 0
            ;;

	# Help Info
        --help|--HELP|-Help|-H|-h)
            display_help;
	      exit 0
            ;;

	# Display the return code messages
	--retcode|--RETCODE|--Retcode|-R|-r)
	      gr_display_generic_header "TEDW Core Tools Patch (Fix Pack)  Installer";
            display_return_code_messages;
	      exit 0
	      ;;

	# Run in Debug
	--debug|--DEBUG|--Debug|-D|-d)
	      DEBUG=defined
	      gr_set_debug_mode;
	      shift
	      continue
	      ;;

	# List installed versions
	--listversions|--LISTVERSIONS|--Listversions|-L|-l)
		echo "    <<< Listing installed Product, Fix Pack, and E-Fix versions only >>>"
            echo ""
	      LIST_INSTALLED=defined; export LIST_INSTALLED
		BYPASS_PASSWORD_REQUIRED=true; export BYPASS_PASSWORD_REQUIRED
	      shift
	      continue
	      ;;

	# Error - not valid option
        -*)
            echo "      (F) CDWIC0080E Unknown Command option ($1)"
            echo "          < Displaying Help >"
            echo ""
            display_help;
            exit 139
            ;;

	# Default
        *)
            echo "      (F) CDWIC0080E Unknown option ($1)"
            echo "          < Displaying Help >"
	      echo ""
            display_help;
            exit 140
            ;;
  esac
done

# Main routines
gr_process_args 2 $args_found;
gr_display_generic_header "TEDW Core Tools Patch (Fix Pack) Installer";
if [ -z "$LIST_INSTALLED" ]; then display_warning; fi
gr_set_debug_mode;
gr_set_interp;
gr_read_config_file;
validate_basic_install;
gr_set_program_vars;
set_private_program_vars;
gr_check_user;
gr_extend_path;


# Execute main routines
if [ -n "$LIST_INSTALLED" ]; then
  exec_list_installed;
else
  execute_install_driver;
fi


clean_up;
post_details;
prep_exit;


exit 0
