#/bin/sh
###################################################################
#
# Tivoli Enterprise Data Warehouse, v 1.1.0
# (C) Copyright IBM Corp. 2002.  All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#
###################################################################
#
# File Information: src/install/bin/twh_core_startup_after.sh, twh_core, twh_1.1_ode
#
# Revision: 1.1
#
# Date: 7/17/02 14:35:40
#
# Desc.  This is the startup driver for the WPack after script.
#        It makes available the appropriate Env Vars and routines
#        so WPacks and execute functions in an after script.
#
# Installation Logs (Default names):
#       A log will be generated for this scirpt called
#       <ava>_after.sh.log
#
#
# Required Env Vars:
#       TWH_TOPDIR
#
# Return Codes (Only!):
#       0 = Success
#       255 = Failure
#
# Interpreter Required:
#       NT/2000 = BASH
#       Unix    = SH (Bourne Shell)
#
#
# Message Types:
#       (I) - Informational
#       (W) - Warning
#       (F) - Fatal
#
#
# Note - Caution needs to be taken for NT paths that have a space.
#        This is a limitation that requires double quoting around all
#        variables that contain a path.  Testing should be done to
#        ensure this script runs with a space in any of the paths 
#        that are used.  For example "c:/program files".
#
#       NOTE - This after script only runs on the Control Server
#              and not on other distributed hosts.
#
###################################################################
#


#  Define values
TRUE=1
FALSE=0

###


# Pulls in the generic shared routines
# Note the first 50 return codes reseved for the gen routines
include_gen_routines()
{
echo "==> Including Generic Routines"
# Set context for generic routines
if [ -z "$TWH_APP_CONTEXT" ]; then
  TWH_APP_CONTEXT=$TRUE; export TWH_APP_CONTEXT
fi

# TWH_TOPDIR needs to always be in the environment
if [ -z "$TWH_TOPDIR" ]; then
  echo "      (F) Could not locate Env var TWH_TOPDIR"
  exit 255
fi

# Set path to gen routines
gen_path="$TWH_TOPDIR/install/bin"

# Confirm file
if [ ! -r "$gen_path"/twh_gen_routines.sh ]; then
  echo "      (F) Could not locate the twh_gen_routines.sh file."
  echo "          Path specified is ($gen_path)"
  exit 255
fi

# pull in the generic routines
. "$gen_path"/twh_gen_routines.sh

echo "      (I) Done including Generic Routines"
return 0;
}


###


#
set_private_program_vars()
{

return 0;
}


###


#
validate_after_env()
{
echo "==> Validating startup environment"

# Check command line args
if [ -z "$TWH_CONFIG_FILE" -o -z "$APP_WORK_DIR" ]; then
  echo "      (F) Not all the required parameters were provided to"
  echo "          this scirpt.  They are:"
  echo "            * Path and name of the TWH Application config file"
  echo "            * Application working directory or install base directory"
  echo ""
  echo "          Usage: <ava>_after.sh name_of_config_file application_install_base_dir"
  echo "          Exiting..."
  echo ""

  exit 255
fi

# TWH_TOPDIR needs to always be in the environment
if [ -z "$TWH_TOPDIR" ]; then
  echo "      (F) Could not locate Env var TWH_TOPDIR"
  exit 255
fi

if [ ! -r "$TWH_CONFIG_FILE" ]; then 
  echo "      (F) Could not read/locate the twh_app_install.cfg file."
  exit 255
fi

if [ ! -r "$APP_WORK_DIR" ]; then 
  echo "      (F) Could not read/locate the application work directory."
  echo "          Should be ($APP_WORK_DIR)"
  exit 255
fi


echo "      (I) Done validating startup environment"
return 0;
}


###

#
exec_core_after()
{
echo ""
echo ""
echo "*********************************"
echo "* Core After Script Start point *"
echo "*********************************"

echo "==> Executing core after script"

# is core after script there
if [ ! -r "$TWH_TOPDIR/install/bin/twh_core_after.sh" ]; then
  echo "      (F) Could not find or execute core after script."
  echo "          File is ($TWH_TOPDIR/install/bin/twh_core_after.sh)."
  echo "          Review application documentation and assure all prerequisites are installed."
  exit 255
fi

"$TWH_TOPDIR/install/bin/twh_core_after.sh" "$TWH_CONFIG_FILE" "$APP_WORK_DIR"
if [ $? -gt 1 ]; then
  echo "      (F) Error detected during the twh_core_after.sh execution."
  echo "          Exiting..."
  exit 255
fi


return 0;
}


########
# Main #
########

exec_startup_driver()
{
echo ""
echo "**  Loading TEDW Local Environment..."
echo ""

# Bring in the generic routines
include_gen_routines;

validate_after_env;
gr_set_interp;
gr_read_config_file;
gr_set_program_vars;
gr_extend_path;
gr_check_user;
gr_test_db2exec;
set_private_program_vars;

echo ""
echo "      **  Done Loading TEDW Local Environment"
echo ""
echo ""

return 0;
}
# End startup driver