#!/bin/sh
#-----------------------------------------------------------------------
#
# Tivoli Enterprise Data Warehouse, v 1.1.2
# (C) Copyright IBM Corp. 2002.  All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#
#
#-----------------------------------------------------------------------
# File Information: src/install/bin/twh_create_wpack_cfg_file.sh, twh_core, twh_1.1_ode, 20021113A
#
# Revision: 1.7
#
# Date: 11/13/02 08:48:06
#-----------------------------------------------------------------------
#

PROGRAM="$0"
config_file="$1"
display_instrs=$2

TRUE=1
FALSE=0

# Pick up the configuration information recorded at initial core install
TWH_CONFIG_FILE="$TWH_TOPDIR/install/logs/twh_core_install.cfg.done"

BYPASS_PASSWORD_REQUIRED=1
export BYPASS_PASSWORD_REQUIRED
TWH_CORE_PATCH_CONTEXT=1	# To read the core patch file
export TWH_CORE_PATCH_CONTEXT

# Pulls in the generic shared routines
# Note return codes 3-70 reserved for the gen routines
include_gen_routines()
{
# Bug on NT - fix the path back to be unix type path
gen_file_and_path=`echo $PROGRAM | sed 's/\\\\/\//g' `

gen_path=`dirname "$gen_file_and_path" `

# Confirm file (twh_gen_routines must be in same directory)
if [ ! -r "$gen_path"/twh_gen_routines.sh ]; then
  echo "      (F) CDWIC0071E Could not locate the twh_gen_routines.sh file."
  exit 71
fi

# pull in the generic routines
. "$gen_path"/twh_gen_routines.sh

return 0;
}


### Main ###

# Source in information from original core install.
include_gen_routines;

gr_display_generic_header "TEDW Warehouse Pack Patch CFG generator";
echo ""
echo "==> No patch config file detected, Auto-Generating for later use..."
echo ""

gr_set_interp;

if [ $UNIX_INTERP -eq 0 ]; then
  TWH_CONFIG_FILE=`echo "$TWH_CONFIG_FILE" | sed 's/\\\\/\//g' `
fi


if [ ! -r "$TWH_CONFIG_FILE" ]; then
  echo "      (F) Config DB file not found."
  echo "          Problem with the TEDW 1.1.0 install."
  echo "          Exiting..."
  exit 1
fi


# Set default for PS
blah=`cat "$TWH_TOPDIR/install/logs/twh_core_comp_list.cfg" | grep "TWH_RPI_COMP=TRUE" 2>&1 `
if [ $? -ne 0 ]; then
  PS_HOME="NULL"
else
  PS_HOME=""
fi


gr_read_config_file;

# save the current directory that the script was run
curdir=`pwd`

config_file=`echo "$config_file" | sed 's/\\\\/\//g'`
PATCH_MEDIA_DIR=`echo $PATCH_MEDIA_DIR | sed 's/\\\\/\//g'`
TWH_TOPDIR=`echo $TWH_TOPDIR | sed 's/\\\\/\//g'`
TMPDIR=`echo $TMPDIR | sed 's/\\\\/\//g'`

echo "==> Writing config file..."


echo "# Tivoli Enterprise Data Warehouse Application Warehouse" > "$config_file"
echo "# Pack Patch Configuration File" >> "$config_file"
echo "#" >> "$config_file"
echo "# (C) COPYRIGHT TIVOLI Systems, Inc. 2002" >> "$config_file"
echo "# Unpublished Work" >> "$config_file"
echo "# All Rights Reserved" >> "$config_file"
echo "# Licensed Material - Property of TIVOLI Systems, Inc." >> "$config_file"
echo "#" >> "$config_file"
echo "###################################################################" >> "$config_file"
echo "#" >> "$config_file"
echo "# Instructions: This config file provides information needed to" >> "$config_file"
echo "# install a patch to one application warehouse enablement pack" >> "$config_file"
echo "# for Tivoli Enterprise Data Warehouse.  You must install this" >> "$config_file"
echo "# patch on each Tivoli Enterprise Data Warehouse system where" >> "$config_file"
echo "# the application warehouse enablement pack was initially" >> "$config_file"
echo "# installed to ensure that the application warehouse enablement" >> "$config_file"
echo "# pack is at the same level on all systems.  Consult the file" >> "$config_file"
echo "# README file 'wep_patch.txt' in the Tivoli Enterprise Data" >> "$config_file"
echo "# Warehouse 1.1 Fix Pack 2 package for further details." >> "$config_file"
echo "#" >> "$config_file"
echo "# 1. Supply correct values to all the property tags listed in the" >> "$config_file"
echo "#    'USER SUPPLIED SECTION'.  These values are not determined" >> "$config_file"
echo "#    when this file is generated and must be supplied.  These" >> "$config_file"
echo "#    values include:" >> "$config_file"
echo "#" >> "$config_file"
echo "#           APP_MEDIA_DIR:    The absolute path to the directory that" >> "$config_file"
echo "#                             contains the patch installation source files." >> "$config_file"
echo "#                             This directory must contain a file with the" >> "$config_file"
echo "#                             name 'twh_install_props.cfg'. " >> "$config_file"
echo "#" >> "$config_file"
echo "#           PS_HOME:          The absolute path to the directory where" >> "$config_file"
echo "#                             Tivoli Presentation Services is installed." >> "$config_file"
echo "#                             This property tag appears only on a system" >> "$config_file"
echo "#                             where the Reporting Interface (RPI)" >> "$config_file"
echo "#                             component is installed." >> "$config_file"
echo "#" >> "$config_file"
echo "#           DB2_PASS:         DB2 administrator password for DB2 on the" >> "$config_file"
echo "#                             current system." >> "$config_file"
echo "#" >> "$config_file"
echo "#           COPT_CTRL_DB2PASS DB2 administrator password for the control" >> "$config_file"
echo "#                             server system." >> "$config_file"
echo "#" >> "$config_file"
echo "#           COPT_CDW_DB2PASS  DB2 administrator password for the central" >> "$config_file"
echo "#                             data warehouse system." >> "$config_file"
echo "#" >> "$config_file"
echo "#           COPT_MART_DB2PASS DB2 administrator password for the data" >> "$config_file"
echo "#                             mart system." >> "$config_file"
echo "#" >> "$config_file"
echo "# 2. Ensure all properties are supplied correctly in the 'AUTO-GENERATED" >> "$config_file"
echo "#    SECTION' and the 'AUTO-GENERATED SECTION - SUPPORT'." >> "$config_file"
echo "#" >> "$config_file"
echo "# 3. The primary file used to auto-generate these properties is " >> "$config_file"
echo "#    $TWH_TOPDIR/install/logs/twh_core_config.cfg.done.  Note that" >> "$config_file"
echo "#    on a single host" >> "$config_file"
echo "#    install all the DB2PASS values will be the same.  Also note that" >> "$config_file"
echo "#    for security, passwords are removed from this file after this " >> "$config_file"
echo "#    patch has been installed.  Additionally if you have problems"  >> "$config_file"
echo "#    installing this patch, you should remove these passwords if you"  >> "$config_file"
echo "#    intend on leaving this configuration file around for some time." >> "$config_file"
echo "#" >> "$config_file"
echo "# NOTES:" >> "$config_file"
echo "# 1. Do not change the values in the 'AUTO-GENERATED SECTION - SUPPORT'"  >> "$config_file"
echo "#    section unless told to do so by your support provider or not"  >> "$config_file"
echo "#    properly auto-generated." >> "$config_file"
echo "# 2. All file paths must be absolute and not relative.  Also all paths" >> "$config_file"
echo "#    must be defined in the Unix format not the Windows format.  This " >> "$config_file"
echo "#    means that c:\temp (on Windows) should be c:/temp with a slash (/)" >> "$config_file"
echo "#    for in the path, not a backslash (\) in the path.  This is the" >> "$config_file"
echo "#    case no matter what operating system you are running this program" >> "$config_file"
echo "#    on." >> "$config_file"
echo "# 3. All properties must have a supplied value and must take the form" >> "$config_file"
echo "#    of property=value exactly.  You cannot have more than one value" >> "$config_file"
echo "#    per property." >> "$config_file"
echo "# 4. Properties that have file paths as values must not include any" >> "$config_file"
echo "#    spaces (whitespace) at the end of that path." >> "$config_file"
echo "#" >> "$config_file"
echo "#" >> "$config_file"
echo "########################################################################" >> "$config_file"
echo "# " >> "$config_file"
echo "############ USER SUPPLIED SECTION #####################################" >> "$config_file"

echo "# Edit the following lines with the information specific to your" >> "$config_file"
echo "# setup. For more information on the definition of each of these" >> "$config_file"
echo "# property tags, please see README file 'wep_patch.txt' in the" >> "$config_file"
echo "# Tivoli Enterprise Data Warehouse 1.1 Fix Pack 2 package." >> "$config_file"
echo "# " >> "$config_file"
echo "APP_MEDIA_DIR=" >> "$config_file"
if [ "$PS_HOME" = "" ]; then
  echo "PS_HOME=" >> "$config_file"
fi
echo "DB2PASS=" >> "$config_file"
echo "COPT_CTRL_DB2PASS=" >> "$config_file"
echo "COPT_CDW_DB2PASS=" >> "$config_file"
echo "COPT_MART_DB2PASS=" >> "$config_file"
echo "# " >> "$config_file"
echo "############ AUTO-GENERATED SECTION #####################################" >> "$config_file"
echo "APP_INSTALL_PROPS_FILE=twh_install_props.cfg" >> "$config_file"
echo "TWH_TOPDIR=$TWH_TOPDIR" >> "$config_file"
echo "TMPDIR=$TMPDIR" >> "$config_file"
if [ "$PS_HOME" = "NULL" ]; then
  echo "PS_HOME=$PS_HOME                    # PS not on system" >> "$config_file"
fi
echo "DB2_INST_DIR=$DB2_INST_DIR" >> "$config_file"
echo "DB2USER=$DB2USER" >> "$config_file"
echo "COPT_CTRL_DB2HOST=$COPT_CTRL_DB2HOST" >> "$config_file"
echo "COPT_CTRL_DB2PORT=$COPT_CTRL_DB2PORT" >> "$config_file"
echo "COPT_CTRL_DB2USER=$COPT_CTRL_DB2USER" >> "$config_file"
echo "COPT_CDW_DB2HOST=$COPT_CDW_DB2HOST" >> "$config_file"
echo "COPT_CDW_DB2PORT=$COPT_CDW_DB2PORT" >> "$config_file"
echo "COPT_CDW_DB2USER=$COPT_CDW_DB2USER" >> "$config_file"
echo "COPT_MART_DB2HOST=$COPT_MART_DB2HOST" >> "$config_file"
echo "COPT_MART_DB2PORT=$COPT_MART_DB2PORT" >> "$config_file"
echo "COPT_MART_DB2USER=$COPT_MART_DB2USER" >> "$config_file"
echo "LOCAL_HOSTNAME=$LOCAL_HOSTNAME" >> "$config_file"
echo "# " >> "$config_file"
echo "############ AUTO-GENERATED SECTION - SUPPORT #####################################" >> "$config_file"
echo "TWH_CTRL_DB=TWH_MD" >> "$config_file"
echo "TWH_CDW_DB=TWH_CDW" >> "$config_file"
echo "TWH_DM_DB=TWH_MART" >> "$config_file"
echo "DB2_INSTALL_LOG=twh_ibm_db2_wpack_patch_runlog.log" >> "$config_file"
echo "TWH_VERSION=1.1.1" >> "$config_file"
echo "TWH_KEEP_TMP_FILES=false" >> "$config_file"
echo "APP_INSTALL_LIST_FILE=NULL" >> "$config_file"
echo "#.done" >> "$config_file"

if [ $display_instrs -eq $TRUE ]; then

   echo
   echo "      (I) Writing config file completed."
   echo "          The config file is located at $config_file"
   echo
   echo "      (I) The following user changeable values are set in the config file:"
   echo ""
   echo "          APP_INSTALL_PROPS_FILE=twh_install_props.cfg"
   echo "          TWH_TOPDIR=$TWH_TOPDIR"
   echo "          TMPDIR=$TMPDIR"
   echo "          DB2_INST_DIR=$DB2_INST_DIR"
   echo "          PS_HOME=$PS_HOME"
   echo "          DB2USER=$DB2USER"
   echo "          DB2PASS="
   echo "          COPT_CTRL_DB2HOST=$COPT_CTRL_DB2HOST"
   echo "          COPT_CTRL_DB2PORT=$COPT_CTRL_DB2PORT"
   echo "          COPT_CTRL_DB2USER=$COPT_CTRL_DB2USER"
   echo "          COPT_CTRL_DB2PASS="
   echo "          COPT_CDW_DB2HOST=$COPT_CDW_DB2HOST"
   echo "          COPT_CDW_DB2PORT=$COPT_CDW_DB2PORT"
   echo "          COPT_CDW_DB2USER=$COPT_CDW_DB2USER"
   echo "          COPT_CDW_DB2PASS="
   echo "          COPT_MART_DB2HOST=$COPT_MART_DB2HOST"
   echo "          COPT_MART_DB2PORT=$COPT_MART_DB2PORT"
   echo "          COPT_MART_DB2USER=$COPT_MART_DB2USER"
   echo "          COPT_MART_DB2PASS="
   echo "          LOCAL_HOSTNAME=$LOCAL_HOSTNAME"
   echo
   echo "==> The config file is located at $config_file"
   echo "==> Please edit this file and fill in the correct information for the"
   echo "    following tags:"
   echo ""
   echo "    APP_MEDIA_DIR"
   if [ "$PS_HOME" = "" ]; then
      echo "    PS_HOME"
   fi
   echo "    DB2PASS"
   echo "    COPT_CTRL_DB2PASS"
   echo "    COPT_CDW_DB2PASS"
   echo "    COPT_MART_DB2PASS"
   echo
   echo "  Once the editing is complete, rerun the tedw_wpack_patchadm.sh script"
   echo "  that was originally run."


   sleep 5

fi
#
