------------------------------------------------------------------------
--
-- Tivoli Enterprise Data Warehouse, v 1.1.0 
-- (C) Copyright IBM Corp. 2002.  All Rights Reserved.
--
-- US Government Users Restricted Rights - Use, duplication or
-- disclosure restricted by GSA ADP Schedule Contract with
-- IBM Corp.
--
--
------------------------------------------------------------------------
--
-- File Information: src/install/sql/clean.sql, twh_core, twh_1.1_ode
--
-- Revision: 1.3.1.1
--
-- Date: 8/22/02 13:20:54
------------------------------------------------------------------------

----------------------------------------------------------
-- dev to test 
----------------------------------------------------------

-- a row is inserted in iwh.iwhstrings for every businessview 
-- that goes from dev to test

delete from iwh.iwhstrings is
where exists (
select 1 from delta.businessview bv
where bv.iwhid=is.iwhid
)
;

----------------------------------------------------------
-- schedules     
----------------------------------------------------------

-- a row is inserted in iwh.relationship for every link from a 
-- businessview to a schedule

delete from iwh.relationship rs
where exists (
select 1 from delta.businessview bv
where 
rs.relation_name='BV_TO_Schedule' and
bv.iwhid=rs.source_iwhid
)
;

-- a row is inserted in iwh.edition for every link from a 
-- businessview to a schedule
 
delete from iwh.edition ed
where exists (
select 1 from delta.businessview bv
where bv.iwhid=ed.businessviewid
)
;

-- new in defect 147643
-- a row is inserted in iwh.relationship for every link from a 
-- process to a schedule

delete from iwh.relationship rs
where exists (
select 1 from delta.process pr
where 
rs.relation_name='Process_TO_BVSchedul' and
pr.iwhid=rs.source_iwhid
)
;

-- new in defect 147643
-- a row is inserted in iwh.edition for every link from a 
-- process to a schedule
 
delete from iwh.edition ed
where exists (
select 1 from delta.process pr
where pr.iwhid=ed.businessviewid
)
;

-- changed in defect 147643 : predicate on rs.relation_name removed
-- delete the schedule if no links to it 

delete from iwh.bvschedule bs
where not exists (
select 1 from iwh.relationship rs
where 
rs.target_iwhid=bs.iwhid
)
;

-- a row is inserted in iwh.binarydata for every process that a 
-- schedule linked with it

delete from iwh.binarydata bd
where exists (
select 1 from delta.process pr
where bd.iwhid=pr.iwhid
)
;

-- a row is inserted in iwh.iwhstrings for every process that a 
-- schedule linked with it

delete from iwh.iwhstrings is
where exists (
select 1 from delta.process pr
where is.iwhid=pr.iwhid
)
;

-------------------------------------------------------
-- Links to groups and users
-------------------------------------------------------

-- a row is inserted in iwh.relationship for every IR src and target 
-- that is associated with a security group
-- Note: the group is not deleted 

delete from iwh.relationship rs
where exists (
select 1 from delta.inforesource ir
where 
rs.relation_name in ('IR_TO_SecGR_Rel','IR_TO_SecGrTar_Rel') and
ir.iwhid=rs.source_iwhid
)
;

-- a row is inserted in iwh.relationship for every Process               
-- that is associated with a security group
-- Note: the group is not deleted 

delete from iwh.relationship rs
where exists (
select 1 from delta.process pr
where 
rs.relation_name = 'Process_TO_SG' and
pr.iwhid=rs.source_iwhid
)
;

-------------------------------------------------------
-- Notifications              
-------------------------------------------------------

-- find the notification steps

create table delta.notifsteps ( iwhid char(10) ) ;

insert into delta.notifsteps
select
distinct r1.target_iwhid as iwhid
from
iwh.relationship r1,
delta.businessview bv
where
r1.relation_name in ('BV_POSTCASCADE_TO_BV','BV_POSTCAS_TO_BVFAIL') and
r1.source_iwhid=bv.iwhid
;

-- find the prog funcs

create table delta.notiffuncs ( iwhid char(10) ) ; 

insert into delta.notiffuncs
select
distinct r1.target_iwhid as iwhid
from
iwh.relationship r1,
delta.notifsteps ns
where
r1.relation_name in ('BV_TO_AggregFunc') and
r1.source_iwhid=ns.iwhid
;

-- find the prog parms

create table delta.notifparms ( iwhid char(10) ) ;

insert into delta.notifparms
select
distinct r1.target_iwhid as iwhid
from
iwh.relationship r1,
delta.notiffuncs nf
where
r1.relation_name in ('ProgParm_TOAggreFunc') and
r1.source_iwhid=nf.iwhid
;

-- delete the prog parms

delete from iwh.progparm pp
where exists (
select 1 from 
delta.notifparms np
where
np.iwhid=pp.iwhid
)
;

-- delete the prog func

delete from iwh.progfunc2 pf
where exists (
select 1 from
delta.notiffuncs nf
where
nf.iwhid=pf.iwhid
)
;

-- delete the business views for the notif steps

delete from iwh.businessview bv
where exists (
select 1 from
delta.notifsteps ns
where
ns.iwhid=bv.iwhid
)
;

-- delete the relationships for the notif funcs
-- delete relationships for the notif prog parms

delete from iwh.relationship rs
where exists ( 
select 1 from
delta.notiffuncs nf 
where
nf.iwhid=rs.source_iwhid
)
;

-- delete the relationships for the notif steps

delete from iwh.relationship rs
where exists (
select 1 from 
delta.notifsteps ns
where
ns.iwhid = rs.source_iwhid or
ns.iwhid = rs.target_iwhid 
)
;





