
----------------------------------------------------------
-- Steps to Schedules     
----------------------------------------------------------

select substr(bv.name,1,45) as step_name, hour, minute, second, 
case 
  when schedtype = 0 then 'Once'
  when schedtype = 1 then 'By Minutes'
  when schedtype = 2 then 'Hourly'
  when schedtype = 3 then 'Daily'
  when schedtype = 4 then 'Weekly'
  when schedtype = 5 then 'Monthly dates'
  when schedtype = 6 then 'Monthly days'
  when schedtype = 7 then 'Yearly'
  else char(schedtype)
end as interval,
case 
  when frequencytype = 1 then 'Every'
  when frequencytype = 2 then 'Every 2'
  when frequencytype = 3 then 'Every 3'
  when frequencytype = 4 then 'Every 4'
  when frequencytype = 5 then 'Every 5'
  when frequencytype = 6 then 'Every 6'
  when frequencytype = 7 then 'Every 7'
  when frequencytype = 8 then 'Every 8'
  else char(frequencytype)
end as frequency,
case 
  when schedtype=4 and dayofweek = 1 then 'Monday'
  when schedtype=4 and dayofweek = 2 then 'Tuesday'
  when schedtype=4 and dayofweek = 3 then 'Wednesday'
  when schedtype=4 and dayofweek = 4 then 'Thursday'
  when schedtype=4 and dayofweek = 5 then 'Friday'
  when schedtype=4 and dayofweek = 6 then 'Saturday'
  when schedtype=4 and dayofweek = 7 then 'Sunday'
  else ''
end as day
from 
delta.businessview bv,
iwh.relationship rs,
iwh.bvschedule bs
where 
rs.relation_name='BV_TO_Schedule' and
bv.iwhid=rs.source_iwhid and
bs.iwhid=rs.target_iwhid
;

----------------------------------------------------------
-- Steps to Notifications              
----------------------------------------------------------

select substr(bv.name,1,45) as step_name, 'On Success' as notify_on 
from
iwh.relationship r1,
delta.businessview bv
where
r1.relation_name in ('BV_POSTCASCADE_TO_BV') and
r1.source_iwhid=bv.iwhid and
not exists (select 1 from iwh.relationship r2
where r2.relation_name = 'BV_POSTCAS_TO_BVFAIL' and
r2.source_iwhid=r1.source_iwhid and
r2.target_iwhid=r1.target_iwhid )
union all
select substr(bv.name,1,45) as step_name, 'On Failure' as notify_on 
from
iwh.relationship r1,
iwh.businessview bv
where
r1.relation_name in ('BV_POSTCAS_TO_BVFAIL') and
r1.source_iwhid=bv.iwhid and
not exists (select 1 from iwh.relationship r2
where r2.relation_name = 'BV_POSTCASCADE_TO_BV' and
r2.source_iwhid=r1.source_iwhid and
r2.target_iwhid=r1.target_iwhid )
union all
select substr(bv.name,1,45) as step_name, 'On Completion' as notify_on 
from
iwh.relationship r1,
iwh.businessview bv
where
r1.relation_name in ('BV_POSTCAS_TO_BVFAIL') and
r1.source_iwhid=bv.iwhid and
exists (select 1 from iwh.relationship r2
where r2.relation_name = 'BV_POSTCASCADE_TO_BV' and
r2.source_iwhid=r1.source_iwhid and
r2.target_iwhid=r1.target_iwhid )
;

----------------------------------------------------------
-- Steps to Agent Sites       
----------------------------------------------------------

select substr(bv.name,1,45) as step_name, substr(ag.name,1,45) as agent_site
from 
delta.businessview bv,
iwh.relationship rs,
iwh.agentsite ag
where 
rs.relation_name='BV_TO_AgentSite' and
ag.iwhid=rs.source_iwhid and
bv.iwhid=rs.target_iwhid and
bv.name not like '%Notification (%'
;

----------------------------------------------------------
-- Warehouse Source/Target to Security Group
----------------------------------------------------------

select substr(ir.name,1,45) as source_or_target, substr(sg.name,1,35) as security_group
from 
iwh.relationship rs,
delta.inforesource ir,
iwh.securitygroup sg
where 
rs.relation_name in ('IR_TO_SecGR_Rel','IR_TO_SecGrTar_Rel') and
ir.iwhid=rs.source_iwhid and
sg.iwhid=rs.target_iwhid
;
 
----------------------------------------------------------
-- Process to Security Group
----------------------------------------------------------

select substr(pr.name,1,45) as process, substr(sg.name,1,35) as security_group
from 
iwh.relationship rs,
delta.process pr,
iwh.securitygroup sg
where 
rs.relation_name = 'Process_TO_SG' and
pr.iwhid=rs.source_iwhid and
sg.iwhid=rs.target_iwhid
;
